'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:02:43 pm'!

TracePosition subclass: #BoundedTrace
	instanceVariableNames: '
		myBranch {BranchDescription}
		myPosition {UInt32}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-Traces'!
BoundedTrace comment:
'Trace Positions represent absolute coordinates in a tree-ordered space.  
The ent navigates to O-roots from H-roots in the northern direction using 
TracePositions.  Refer to the Ent document for a description of the current 
implementation.  TracePositions are generated by asking another tracePosition
for a newSuccesor.  The starting tracePosition is the initil TraceTree.  It 
corresponds to branch 1, position 1.'!

(BoundedTrace getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #COPY; yourself)!

!BoundedTrace methodsFor: 'testing'!

{UInt32} actualHashForEqual
	"This isn't used now.  It might be later.  Hash 
	 based on both the branch and position  This hash
	 uses a couple of arbitrary primes."

	^(myBranch hashForEqual + myPosition * 10993) bitAnd: 134217727 "2^27-1"!

{BooleanVar} isEqual: another {Heaper}

	another
		cast: BoundedTrace into: [:bt |
			^(myBranch isEqual: bt branch)
			 and: [myPosition == bt position]]
		others: [^false].
	^false "fodder"!

{BooleanVar} isLE: another {TracePosition}
	"Return true if the two positions are comparable and the receiver is  
	less than the argument. The argument is put in the cache so that  
	many comparisons IN THE SAME ORDER will occur very fast. This  
	could probably check for both tracePositions in the cache for speed  
	even when the arguments are swapped."

	^myBranch does: myPosition include: another! !

!BoundedTrace methodsFor: 'accessing'!

{BranchDescription} branch
	"This should only be accessible in this module."

	^myBranch!

{void} cacheIn: navCache {PrimIndexTable}
	"Cache the nav-data for the receiver in navCache."

	| oldValue {IntegerVar} | 
	oldValue _ navCache fetch: myBranch.
	oldValue == -1
		ifTrue: 
			[navCache at: myBranch store: myPosition.
			myBranch cacheRecur: navCache]
		ifFalse: 
			[navCache at: myBranch store: (oldValue max: myPosition)].!

{TracePosition} newSuccessor
	"Return a new successor to the receiver. The first successor is on the same 
	branch with a higher position. Further successors are alllocated in a 
	binary-tree fashion along a new branch."
	self knownBug.
	"This consistent count is upperbound over 12 runs of works test.
	It does not seem to be bound."
	DiskManager consistent: 14 with:
		[^myBranch createAfter: self]!

{TracePosition} newSuccessorAfter: trace {TracePosition} 
	"Return a new tracePosition that is after both the receiver and tracePos."

	self knownBug.
	"This consistent count is upperbound over 12 runs of works test.
	It does not seem to be bound."
	DiskManager consistent: 14 with:
		[| branch {BranchDescription} |
		branch _ myBranch makeBranch: self with: trace.
		myBranch installBranch: branch after: self.
		(trace quickCast: BoundedTrace) branch installBranch: branch after: trace.
		^branch nextPosition]!

{UInt32} position
	"This should only be accessible in this module."

	^myPosition!

{ImmuSet of: TracePosition} successors
	"Return all the successors of the receiver."

	^myBranch successorsOf: self! !

!BoundedTrace methodsFor: 'smalltalk:'!

{void} inspect
	| seen |
	seen _ Set new.
	EntView openOn: (TreeBarnacle new
			buildOn: self
			gettingChildren:
				[:tracePos | 
				(seen includes: tracePos)
					ifTrue: [OrderedCollection new]
					ifFalse: [seen add: tracePos.
							tracePos successors asOrderedCollection]]
			gettingImage: [:tracePos | tracePos printString asDisplayText]
			at: 0 @ 0
			vertical: false
			separation: 10 @ 10)!

{void} printOn: aStream {Stream}
	(myPosition = 1 and: [myBranch isKindOf: DagBranch]) ifTrue: [aStream << '>'].
	aStream << myBranch << '.' << myPosition! !

!BoundedTrace methodsFor: 'create'!

create: br {BranchDescription} with: pos {UInt32}
	super create.
	myBranch _ br.
	myPosition _ pos! !

!BoundedTrace methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	myBranch _ receiver receiveHeaper.
	myPosition _ receiver receiveUInt32.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	xmtr sendHeaper: myBranch.
	xmtr sendUInt32: myPosition.! !