'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 8:59:45 pm'!

Position subclass: #ID
	instanceVariableNames: '
		mySpace {IDSpace | NULL}
		myBackend {Sequence | NULL}
		myNumber {IntegerVar}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-Be-Basic'!
ID comment:
'Implementation note:

An ID exists within a particular IDSpace, and is generated by a particular Server. It holds onto the space and the Server which created it, along with a number identifying the ID uniquely. If mySpace is NULL, then the ID is in the global IDSpace. If myBackend is NULL, then this ID was generated by the current Server (unless myNumber is negative, in which case it is considered to have been generated by the "global" backend). If myBackend is non-NULL, then myNumber must be non-negative.'!

(ID getOrMakeCxxClassDescription)
	friends:
'friend class IDRegion;
friend class IDStepper;
friend class IDUpOrder;
friend class IDTester;
friend class IDSpace;
';
	attributes: ((Set new) add: #CONCRETE; add: #ON.CLIENT; add: #COPY; yourself)!

!ID methodsFor: 'accessing'!

{XnRegion} asRegion

	myBackend == NULL ifTrue:
		[^IDRegion make: mySpace
			with: (IntegerRegion make: myNumber)
			with: NULL
			with: false]
	ifFalse:
		[ | others {MuTable of: Sequence and: IntegerRegion} |
		others := MuTable make: SequenceSpace make.
		others at: myBackend introduce: (IntegerRegion make: myNumber).
		^IDRegion make: mySpace
			with: IntegerRegion make
			with: others asImmuTable
			with: false].!

{CoordinateSpace} coordinateSpace

	mySpace == NULL ifTrue:
		[^IDSpace global].
	^mySpace!

{UInt8Array CLIENT} export
	"Essential. Export this iD in a form which can be handed to Server::importID on any Server to generate the same ID"
	
	| xmtr {SpecialistXmtr} result {WriteVariableArrayStream} |
	result := WriteVariableArrayStream make: 200.
	xmtr := Binary2XcvrMaker make
		makeXmtr: (TransferSpecialist make: Cookbook make)
		with: result.
	ID exportSequence: xmtr
		with: (self coordinateSpace cast: IDSpace) backend.
	xmtr  sendIntegerVar: (self coordinateSpace cast: IDSpace) spaceNumber.
	ID exportSequence: xmtr
		with: self backend.
	xmtr sendIntegerVar: self number.
	^result array! !

!ID methodsFor: 'comparing'!

{UInt32} actualHashForEqual

	| result {UInt32} |
	result := self getCategory hashForEqual.
	mySpace ~~ NULL ifTrue:
		[result := result bitXor: mySpace hashForEqual].
	myBackend ~~ NULL ifTrue:
		[result := result bitXor: myBackend hashForEqual].
	^result bitXor: myNumber DOThashForEqual!

{BooleanVar} isEqual: heaper {Heaper}

	heaper cast: ID into: [ :other |
		mySpace == NULL
				ifTrue: [other fetchSpace == NULL ifFalse: [^false]]
				ifFalse: [(other fetchSpace ~~ NULL
					and: [mySpace isEqual: other fetchSpace]) ifFalse: [^false]].
		myBackend == NULL
				ifTrue: [other fetchBackend == NULL ifFalse: [^false]]
				ifFalse: [(other fetchBackend ~~ NULL
					and: [myBackend isEqual: other fetchBackend]) ifFalse: [^false]].
		^ myNumber = other number]
	others:
		[^false].
	^false "fodder"! !

!ID methodsFor: 'protected: create'!

create: space {IDSpace | NULL} with: backend {Sequence | NULL} with: number {IntegerVar}

	super create.
	mySpace := space.
	myBackend := backend.
	myNumber := number.! !

!ID methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	oo << (self coordinateSpace cast: IDSpace) identifier << ':' << self identifier! !

!ID methodsFor: 'private:'!

{Sequence} backend
	"Essential. A Sequence identifying the server on which this was created"
	
	myBackend == NULL
		ifTrue: [myNumber < IntegerVarZero
			ifTrue: [^Sequence zero]
			ifFalse: [^FeServer identifier]]
		ifFalse: [^myBackend]!

{Sequence | NULL} fetchBackend

	^myBackend!

{IDSpace | NULL} fetchSpace

	^mySpace!

{IntegerVar} number
	"Essential. The number identifying this ID from all others generated by the same Server in the same IDSpace."
	
	^myNumber! !

!ID methodsFor: 'obsolete:'!

{Sequence} identifier
	"A sequence of numbers which uniquely identify this ID within its space"
	
	Ravi thingToDo. "get rid of this, and clients"
	^self backend withLast: myNumber! !

!ID methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	mySpace _ receiver receiveHeaper.
	myBackend _ receiver receiveHeaper.
	myNumber _ receiver receiveIntegerVar.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	xmtr sendHeaper: mySpace.
	xmtr sendHeaper: myBackend.
	xmtr sendIntegerVar: myNumber.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ID class
	instanceVariableNames: ''!

(ID getOrMakeCxxClassDescription)
	friends:
'friend class IDRegion;
friend class IDStepper;
friend class IDUpOrder;
friend class IDTester;
friend class IDSpace;
';
	attributes: ((Set new) add: #CONCRETE; add: #ON.CLIENT; add: #COPY; yourself)!

!ID class methodsFor: 'module private create'!

make: space {IDSpace | NULL} with: backend {Sequence | NULL} with: number {IntegerVar}
	^ self create: space with: backend with: number! !

!ID class methodsFor: 'private: pseudo constructors'!

{ID} using: space {IDSpace | NULL} with: backend {Sequence | NULL} with: number {IntegerVar}
	"Special for IDStepper - checks whether it should make backend be NULL"
	[BeGrandMap] USES.
	((backend == NULL
			or: [backend isEqual: Sequence zero])
			or: [backend isEqual: FeServer identifier]) 
		ifTrue: [^self make: space with: NULL with: number]
		ifFalse: [^self make: space with: backend with: number]! !

!ID class methodsFor: 'smalltalk: passe'!

{ID} key: string {char star}
	"ID key: 'test'"
	self passe.!

make: pakobits {PackOBits}

	self passe.!

make: left {IntegerVar} with: right {IntegerVar}
	self passe.! !

!ID class methodsFor: 'smalltalk: system'!

info.stProtocol
"{UInt8Array CLIENT} export
"! !

!ID class methodsFor: 'creation'!

{ID CLIENT login} import: data {PrimIntArray}
	"Essential. Take some information describing an ID and create the ID it was exported from."
	
	| rcvr {SpecialistRcvr} spaceBackend {Sequence} spaceNumber {IntegerVar}
	  iDBackend {Sequence} iDNumber {IntegerVar} space {IDSpace} |
	rcvr := Binary2XcvrMaker make
		makeRcvr: (TransferSpecialist make: Cookbook make)
		with: (XnReadStream make: (data cast: UInt8Array)).
	spaceBackend := self importSequence: rcvr.
	spaceNumber := rcvr receiveIntegerVar.
	iDBackend := self importSequence: rcvr.
	iDNumber := rcvr receiveIntegerVar.
	space := IDSpace make: spaceBackend with: spaceNumber.
	(space isEqual: CurrentGrandMap fluidGet globalIDSpace) ifTrue:
		[space := NULL].
	^ID using: space with: iDBackend with: iDNumber! !

!ID class methodsFor: 'private: export/import for friends'!

{void} exportIntegerRegion: xmtr {SpecialistXmtr} with: integers {IntegerRegion}
	"Write a IntegerRegion onto a stream"
	
	xmtr sendIntegerVar: integers isBoundedBelow not.
	xmtr sendIntegerVar: integers secretTransitions count.
	Int32Zero almostTo: integers secretTransitions count do: [ :i {Int32} |
		xmtr sendIntegerVar: (integers secretTransitions integerAt: i)]!

{void} exportSequence: xmtr {SpecialistXmtr} with: sequence {Sequence}
	"Write a Sequence onto a stream"
	
	sequence isZero ifTrue:
		[xmtr sendIntegerVar: IntegerVarZero.
		^VOID].
	xmtr sendIntegerVar: sequence lastIndex - sequence firstIndex + 1.
	xmtr sendIntegerVar: sequence firstIndex.
	sequence firstIndex to: sequence lastIndex do: [ :i {IntegerVar} |
		xmtr sendIntegerVar: (sequence integerAt: i)].!

{IntegerRegion} importIntegerRegion: rcvr {SpecialistRcvr}
	"Read a IntegerRegion from a stream"
	
	| startsInside {BooleanVar} n {Int32} transitions {IntegerVarArray} |
	startsInside := rcvr receiveIntegerVar DOTasLong.
	n := rcvr receiveIntegerVar DOTasLong.
	transitions := IntegerVarArray zeros: n.
	Int32Zero almostTo: n do: [ :i {Int32} |
		transitions at: i storeInteger: rcvr receiveIntegerVar].
	^IntegerRegion using: startsInside with: n with: transitions!

{Sequence} importSequence: rcvr {SpecialistRcvr}
	"Read a Sequence from a stream"
	
	| count {IntegerVar} shift {IntegerVar} numbers {IntegerVarArray} |
	count := rcvr receiveIntegerVar.
	count == IntegerVarZero ifTrue:
		[^Sequence zero].
	numbers := IntegerVarArray zeros: count DOTasLong.
	shift := rcvr receiveIntegerVar.
	Int32Zero almostTo: count DOTasLong do: [ :i {Int32} |
		numbers at: i storeInteger: rcvr receiveIntegerVar].
	^SequenceSpace make position: numbers with: shift! !