'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:01:20 pm'!

XnRegion subclass: #IDRegion
	instanceVariableNames: '
		mySpace {IDSpace | NULL}
		myLocalIDs {IntegerRegion}
		myImportedIDs {ImmuTable | NULL of: Sequence and: IntegerRegion}
		myIncludesRest {BooleanVar}'
	classVariableNames: '
		TheGlobalNumbers {IntegerRegion} 
		TheLocalNumbers {IntegerRegion} '
	poolDictionaries: ''
	category: 'Xanadu-id'!
IDRegion comment:
'If mySpace is NULL, then it is assumed to be global IDSpace.
The non-negative part of myLocalIDs contains the number portion of all IDs in the region that were generated by the current backend. The negative part contains all IDs which were generated by the "global" backend.
If myImportedIDs is NULL, it is considered to be empty. If it is non-NULL, it must be non-empty. It contains the numbers of all IDs in this region which were generated by other backends. If an entry in the table could be omitted without effect, then it should be omitted. (i.e. if myIncludesRest and the region is full, or myIncludesRest not and the region is empty, then the entry should be omitted).
If myIncludesRest is true, then the region includes, in addition to those IDs explictly listed in myLocalIDs and myImportedIDs, all ID generated by all other backends.'!

(IDRegion getOrMakeCxxClassDescription)
	friends:
'friend class IDSpace;
friend class IDSimpleStepper;
friend class IDStepper;
friend class IDUpOrder;
';
	attributes: ((Set new) add: #CONCRETE; add: #ON.CLIENT; add: #COPY; yourself)!

!IDRegion methodsFor: 'accessing'!

{XnRegion} asSimpleRegion

	Ravi thingToDo. "make this more efficient and return fullRegion less often"
	self isSimple ifTrue:
		[^self].
	myImportedIDs ~~ NULL ifTrue:
		[^self coordinateSpace fullRegion].
	^IDRegion make: mySpace
		with: (myLocalIDs asSimpleRegion cast: IntegerRegion)
		with: NULL
		with: myIncludesRest!

{CoordinateSpace} coordinateSpace

	mySpace == NULL
		ifTrue: [^IDSpace global].
	^mySpace!

{UInt8Array CLIENT} export
	"Essential. Export the IDRegion in a form that can be used to recreate it with IDRegion::import."
		
	| xmtr {SpecialistXmtr} result {WriteVariableArrayStream} |
	result := WriteVariableArrayStream make: 500.
	xmtr := Binary2XcvrMaker make
		makeXmtr: (TransferSpecialist make: Cookbook make)
		with: result.
	ID exportSequence: xmtr
		with: (self coordinateSpace cast: IDSpace) backend.
	ID exportIntegerRegion: xmtr with: myLocalIDs.
	myImportedIDs == NULL ifTrue:
		[xmtr sendIntegerVar: IntegerVarZero]
	ifFalse:
		[xmtr sendIntegerVar: myImportedIDs count.
		myImportedIDs stepper forPositions: [ :key {Sequence} :value {IntegerRegion} |
			ID exportSequence: xmtr with: key.
			ID exportIntegerRegion: xmtr with: value]].
	xmtr sendIntegerVar: myIncludesRest.
	^result array!

{Position} theOne

	myIncludesRest ifFalse:
		[myLocalIDs isEmpty ifTrue:
			[myImportedIDs ~~ NULL ifTrue:
				[^ID make: mySpace
					with: (myImportedIDs domain theOne cast: Sequence)
					with: ((myImportedIDs theOne cast: IntegerRegion) theOne cast: IntegerPos) asIntegerVar]]
		ifFalse:
			[myImportedIDs == NULL ifTrue:
				[^ID make: mySpace
					with: NULL
					with: (myLocalIDs theOne cast: IntegerPos) asIntegerVar]]].
	Heaper BLAST: #NotOneElement.
	^NULL! !

!IDRegion methodsFor: 'testing'!

{UInt32} actualHashForEqual

	| result {UInt32} |
	result := self getCategory hashForEqual bitXor: myLocalIDs hashForEqual.
	mySpace ~~ NULL ifTrue:
		[result := result bitXor: mySpace hashForEqual].
	myImportedIDs ~~ NULL ifTrue:
		[result := result bitXor: myImportedIDs hashForEqual].
	myIncludesRest ifTrue:
		[result := result bitXor: 65535].
	^result!

{BooleanVar} hasMember: position {Position} 

	position cast: ID into: [ :iD | | be {Sequence | NULL} region {IntegerRegion | NULL} |
		be := iD fetchBackend.
		be == NULL
			ifTrue: [^myLocalIDs hasIntMember: iD number ]
			ifFalse:
				[myImportedIDs ~~ NULL
					ifTrue: [region := (myImportedIDs fetch: be) cast: IntegerRegion]
					ifFalse: [region := NULL].
				region == NULL
					ifTrue: [^myIncludesRest]
					ifFalse: [^region hasIntMember: iD number]]].
	^false "fodder"!

{BooleanVar} isEmpty

	^myIncludesRest not
		and: [myImportedIDs == NULL
		and: [myLocalIDs isEmpty]]!

{BooleanVar} isEqual: other {Heaper}

	other cast: IDRegion into: [ :iDs |
		mySpace == NULL
				ifTrue: [iDs fetchSpace == NULL ifFalse:[^false]]
				ifFalse: [(iDs fetchSpace ~~ NULL
					and: [iDs fetchSpace isEqual: mySpace]) ifFalse:[^false]].
		(myIncludesRest == iDs includesRest and: [myLocalIDs isEqual: iDs localIDs])
			ifTrue: [myImportedIDs == NULL
				ifTrue: [^ iDs fetchImportedIDs == NULL]
				ifFalse: [^ iDs fetchImportedIDs ~~ NULL
					and: [iDs fetchImportedIDs isEqual: myImportedIDs]]].
		^false]
	others:
		[^false].
	^false "fodder"!

{BooleanVar} isFinite

	(myIncludesRest not and: [myLocalIDs isFinite]) ifFalse: [^false].
	myImportedIDs ~~ NULL ifTrue:
		[myImportedIDs stepper forEach: [:numbers {IntegerRegion} |
			numbers isFinite ifFalse: [^false]]].
	^true!

{BooleanVar} isFull

	^myImportedIDs == NULL
		and: [myIncludesRest
		and: [myLocalIDs isFull]]!

{BooleanVar} isSimple

	myImportedIDs == NULL ifTrue:
		[^myLocalIDs isSimple].
	myIncludesRest ifTrue:
		[myLocalIDs isSimple ifFalse: [^false].
		myImportedIDs stepper forEach: [ :iDs {IntegerRegion} |
			iDs isSimple ifFalse: [^false]].
		^true]
	ifFalse:
		[^myLocalIDs isEmpty
			and: [myImportedIDs count = 1
			and: [(myImportedIDs theOne cast: IntegerRegion) isSimple]]]!

{BooleanVar} isSubsetOf: region {XnRegion}
	
	region cast: IDRegion into: [ :other |
		(myLocalIDs isSubsetOf: other localIDs)
			ifFalse: [^false].
		myImportedIDs == NULL ifTrue:
			[^myIncludesRest not
				or: [other includesRest
					and: [other fetchImportedIDs == NULL]]].
		other fetchImportedIDs == NULL ifTrue:
			[^other includesRest "since we know I have imported IDs"].
		myIncludesRest ifTrue:
			[(other includesRest
					and: [other fetchImportedIDs domain isSubsetOf: myImportedIDs domain])
				ifFalse: [^false]].
		myImportedIDs stepper forPositions:
			[ :backend {Sequence} :iDs {IntegerRegion} |
			| otherIDs {IntegerRegion | NULL} |
			otherIDs := (other getImportedIDs fetch: backend) cast: IntegerRegion.
			otherIDs == NULL
				ifTrue: [(other includesRest or: [iDs isEmpty])
					ifFalse: [^false]]
				ifFalse: [(iDs isSubsetOf: otherIDs)
					ifFalse: [^false]]]].
	^true! !

!IDRegion methodsFor: 'protected: enumerating'!

{Stepper of: Position} actualStepper: order {OrderSpec default: NULL} 

	self knownBug. "might be enumerable in other cases"
	self isFinite ifFalse:
		[Heaper BLAST: #NotEnumerable].
	^IDStepper create: self! !

!IDRegion methodsFor: 'enumerating'!

{IntegerVar} count

	| result {IntegerVar} |
	myIncludesRest ifTrue:
		[Heaper BLAST: #MustBeFinite].
	result := myLocalIDs count.
	myImportedIDs ~~ NULL ifTrue:
		[myImportedIDs stepper forEach: [ :iDs {IntegerRegion} |
			result := result + iDs count]].
	^result!

{ScruSet of: XnRegion} distinctions

	| result {SetAccumulator} table {MuTable of: Sequence} |
	Ravi thingToDo. "consolidate duplicated code"
	result := SetAccumulator make.
	myImportedIDs == NULL ifTrue:
		[myLocalIDs distinctions stepper forEach: [ :local {IntegerRegion} |
			| region {IDRegion} |
			region _ IDRegion make: mySpace
				with: local with: NULL with: myIncludesRest.
			result step: region]]
	ifFalse:
		[myIncludesRest ifTrue:
			[myLocalIDs distinctions stepper forEach:
				[ :local {IntegerRegion} |
				| region {IDRegion} |
				region _ IDRegion make: mySpace
									with: local with: NULL with: true.
				result step: region].
			myImportedIDs stepper forPositions:
				[ :backend {Sequence} :iDs {IntegerRegion} |
				iDs distinctions stepper forEach:
					[ :import {IntegerRegion} |
					| region {IDRegion} |
					table := MuTable make: SequenceSpace make.
					table at: backend store: import.
					region _ IDRegion make: mySpace
										with: IntegerRegion allIntegers
										with: table asImmuTable
										with: true.
					result step: region]]]
		ifFalse:
			[ | backend {Sequence} |
			(myLocalIDs isEmpty and: [myImportedIDs count = 1]) ifFalse:
				[Heaper BLAST: #MustBeSimple].
			backend := myImportedIDs domain theOne cast: Sequence.
			(myImportedIDs theOne cast: IntegerRegion) distinctions stepper forEach:
				[ :import {Sequence} |
				| region {IDRegion} |
				table := MuTable make: SequenceSpace make.
				table at: backend store: import.
				region _ IDRegion make: mySpace
						with: myLocalIDs
						with: table asImmuTable
						with: false.
				result step: region]]].
	^result value cast: ScruSet!

{Stepper} simpleRegions: order {OrderSpec default: NULL} 

	^IDSimpleStepper create: self! !

!IDRegion methodsFor: 'private:'!

{SequenceRegion} backends
	"All backends which have generated IDs in this Region"
	
	| result {XnRegion} |
	myIncludesRest ifTrue:
		[result := SequenceSpace make fullRegion.
		(myLocalIDs intersects: TheGlobalNumbers) ifFalse:
			[result := result without: Sequence zero].
		(myLocalIDs intersects: TheLocalNumbers) ifFalse:
			[result := result without: FeServer identifier].
		myImportedIDs ~~ NULL ifTrue:
			[myImportedIDs stepper forPositions: [ :backend {Sequence} :iDs {IntegerRegion} |
				iDs isEmpty ifTrue:
					[result := result without: backend]]].
		^result cast: SequenceRegion]
	ifFalse:
		[result := SequenceSpace make emptyRegion.
		(myLocalIDs intersects: TheGlobalNumbers) ifTrue:
			[result := result with: Sequence zero].
		(myLocalIDs intersects: TheLocalNumbers) ifTrue:
			[result := result with: FeServer identifier].
		myImportedIDs ~~ NULL ifTrue:
			[result := result unionWith: myImportedIDs domain].
		^result cast: SequenceRegion]!

{XnRegion of: Sequence} explicitBackends
	"All backends which are non-empty and are explicitly listed. For IDSimpleStepper"
	
	| result {XnRegion} |
	result := SequenceSpace make emptyRegion.
	(myLocalIDs intersects: TheGlobalNumbers) ifTrue:
		[result := result with: Sequence zero].
	(myLocalIDs intersects: TheLocalNumbers) ifTrue:
		[result := result with: FeServer identifier].
	myImportedIDs ~~ NULL ifTrue:
		[myIncludesRest ifTrue:
			[myImportedIDs stepper forPositions:
				[ :backend {Sequence} :iDs {IntegerRegion} |
				iDs isEmpty ifFalse:
					[result := result with: backend]]]
		ifFalse:
			[result := result unionWith: myImportedIDs domain]].
	^result!

{ImmuTable | NULL of: Sequence and: IntegerRegion} fetchImportedIDs

	^myImportedIDs!

{IDRegion} fetchInexplicit
	"The region which covers material not in the explicitBackends list, or NULL if there is none."
	
	| result {MuTable | NULL} actualResult {ImmuTable | NULL} |
	myIncludesRest ifFalse:
		[^NULL].
	myImportedIDs == NULL ifTrue:
		[result := NULL]
	ifFalse:
		[result := MuTable make: SequenceSpace make.
		myImportedIDs stepper forPositions: [ :backend {Sequence} :iDs {IntegerRegion} |
			result at: backend introduce: IntegerRegion make]].
	result ~~ NULL ifTrue: [actualResult := result asImmuTable] ifFalse: [actualResult := NULL].
	^IDRegion make: mySpace
		with: IntegerRegion make
		with: actualResult
		with: true!

{IDSpace} fetchSpace

	^mySpace!

{ImmuTable | NULL of: Sequence and: IntegerRegion} getImportedIDs

	myImportedIDs == NULL ifTrue:
		[Heaper BLAST: #InvalidRequest].
	^myImportedIDs!

{XnRegion of: IntegerPos} iDNumbersFrom: backend {Sequence}
	"The numbers of all IDs in this region that were generated by the given backend"
	
	| result {XnRegion} |
	(backend isEqual: Sequence zero) ifTrue:
		[^myLocalIDs intersect: TheGlobalNumbers].
	(backend isEqual: FeServer identifier) ifTrue:
		[^myLocalIDs intersect: TheLocalNumbers].
	myImportedIDs ~~ NULL ifTrue:
		[result := (myImportedIDs fetch: backend) cast: XnRegion.
		result ~~ NULL ifTrue:
			[^result]].
	myIncludesRest
		ifTrue: [^TheLocalNumbers]
		ifFalse: [^IntegerSpace make emptyRegion]!

{BooleanVar} includesRest

	^myIncludesRest!

{IntegerRegion} localIDs

	^myLocalIDs! !

!IDRegion methodsFor: 'operations'!

{XnRegion} complement

	| result {MuTable of: Sequence and: IntegerRegion} resTable {ImmuTable} |
	myImportedIDs == NULL ifTrue:
		[result := NULL]
	ifFalse:
		[result := MuTable make: SequenceSpace make.
		myImportedIDs stepper forPositions:
			[ :backend {Sequence} :iDs {IntegerRegion} |
			result at: backend store: (TheLocalNumbers minus: iDs)]].
	result == NULL ifTrue: [resTable := NULL] ifFalse: [resTable := result asImmuTable].
	^IDRegion make: mySpace
		with: (myLocalIDs complement cast: IntegerRegion)
		with: resTable
		with: myIncludesRest not!

{XnRegion} intersect: region {XnRegion} 
	| resTable {ImmuTable} |
	region cast: IDRegion into: [ :other | | result {MuTable} |
		myImportedIDs == NULL ifTrue:
			[(myIncludesRest and: [other fetchImportedIDs ~~ NULL])
				ifTrue: [result := other fetchImportedIDs asMuTable]
				ifFalse: [result := NULL]]
		ifFalse:
			[other fetchImportedIDs == NULL ifTrue:
				[other includesRest
					ifTrue: [result := myImportedIDs asMuTable]
					ifFalse: [result := NULL]]
			ifFalse:
				[result := MuTable make: SequenceSpace make.
				myImportedIDs stepper forPositions:
					[ :backend {Sequence} :iDs {IntegerRegion} |
					| otherIDs {IntegerRegion} |
					otherIDs := (other getImportedIDs fetch: backend) cast: IntegerRegion.
					otherIDs ~~ NULL ifTrue:
						[result at: backend store: (iDs intersect: otherIDs)]
					ifFalse: [other includesRest ifTrue:
						[result at: backend store: iDs]]].
				myIncludesRest ifTrue:
					[other getImportedIDs stepper forPositions:
						[ :backend {Sequence} :otherIDs {IntegerRegion} |
						| iDs {IntegerRegion} |
						iDs := (myImportedIDs fetch: backend) cast: IntegerRegion.
						iDs == NULL ifTrue:
							[result at: backend store: otherIDs]]]]].
		result ~~ NULL ifTrue:
			[result stepper forPositions:
				[ :backend {Sequence} :iDs {IntegerRegion} | | f {BooleanVar} |
				(myIncludesRest and: [other includesRest])
					ifTrue: [f := iDs isEqual: TheLocalNumbers]
					ifFalse: [f := iDs isEmpty].
				f ifTrue: [result wipe: backend]].
			result isEmpty ifTrue:
				[result := NULL]].
		result == NULL ifTrue: [resTable := NULL] ifFalse: [resTable := result asImmuTable].
		^IDRegion make: mySpace
			with: ((myLocalIDs intersect: other localIDs) cast: IntegerRegion)
			with: resTable
			with: (myIncludesRest and: [other includesRest])].
	^NULL "fodder"!

{XnRegion} simpleUnion: region {XnRegion} 

	region cast: IDRegion into: [ :other |
		Ravi thingToDo. "return fullRegion less often"
		(myImportedIDs ~~ NULL or: [other fetchImportedIDs ~~ NULL])
			ifTrue: [^self coordinateSpace fullRegion].
		^IDRegion make: mySpace
			with: ((myLocalIDs simpleUnion: other localIDs) cast: IntegerRegion)
			with: NULL
			with: myIncludesRest].
	^NULL "fodder"!

{XnRegion} unionWith: region {XnRegion} 
	| resTable {ImmuTable} |
	region cast: IDRegion into: [ :other | | result {MuTable} |
		myImportedIDs == NULL ifTrue:
			[(myIncludesRest or: [other fetchImportedIDs == NULL])
				ifTrue: [result := NULL]
				ifFalse: [result := other fetchImportedIDs asMuTable]]
		ifFalse:
			[other fetchImportedIDs == NULL ifTrue:
				[other includesRest
					ifTrue: [result := NULL]
					ifFalse: [result := myImportedIDs asMuTable]]
			ifFalse:
				[result := MuTable make: SequenceSpace make.
				myImportedIDs stepper forPositions:
					[ :backend {Sequence} :iDs {IntegerRegion} |
					| otherIDs {IntegerRegion} |
					otherIDs := (other getImportedIDs fetch: backend) cast: IntegerRegion.
					otherIDs ~~ NULL ifTrue:
						[result at: backend store: (iDs unionWith: otherIDs)]
					ifFalse: [other includesRest ifFalse:
						[result at: backend store: iDs]]].
				myIncludesRest ifFalse:
					[other getImportedIDs stepper forPositions:
						[ :backend {Sequence} :otherIDs {IntegerRegion} |
						| iDs {IntegerRegion} |
						iDs := (myImportedIDs fetch: backend) cast: IntegerRegion.
						iDs == NULL ifTrue:
							[result at: backend store: otherIDs]]]]].
		result ~~ NULL ifTrue:
			[result stepper forPositions:
				[ :backend {Sequence} :iDs {IntegerRegion} | | f {BooleanVar} |
				(myIncludesRest or: [other includesRest])
					ifTrue: [f := iDs isEqual: TheLocalNumbers]
					ifFalse: [f := iDs isEmpty].
				f ifTrue: [result wipe: backend]].
			result isEmpty ifTrue:
				[result := NULL]].
		result == NULL ifTrue: [resTable := NULL] ifFalse: [resTable := result asImmuTable].
		^IDRegion make: mySpace
			with: ((myLocalIDs unionWith: other localIDs) cast: IntegerRegion)
			with: resTable
			with: (myIncludesRest or: [other includesRest])].
	^NULL "fodder"!

{XnRegion} with: pos {Position} 
	| resTable {ImmuTable} newLocalIDs {IntegerRegion} |
	pos cast: ID into: [ :id | | result {MuTable} |
		myImportedIDs == NULL ifTrue:
			[(myIncludesRest or: [id fetchBackend == NULL])
				ifTrue: [result := NULL]
				ifFalse: [result := MuTable make: id fetchSpace.
						result at: id fetchBackend introduce: (IntegerRegion make: id number)]]
		ifFalse:
			[id fetchBackend == NULL ifTrue:
				[result := myImportedIDs asMuTable]
			ifFalse:
				[result := MuTable make: SequenceSpace make.
				myImportedIDs stepper forPositions:
					[ :backend {Sequence} :iDs {IntegerRegion} |
					(id fetchBackend isEqual: backend)
						ifTrue: [result at: backend store: (iDs withInt: id number)]
						ifFalse: [result at: backend store: iDs]]].
				myIncludesRest ifFalse:
					[result at: id fetchBackend store: (IntegerRegion make: id number)]].
		result ~~ NULL ifTrue:
			[result stepper forPositions:
				[ :backend {Sequence} :iDs {IntegerRegion} | | f {BooleanVar} |
				myIncludesRest
					ifTrue: [f := iDs isEqual: TheLocalNumbers]
					ifFalse: [f := iDs isEmpty].
				f ifTrue: [result wipe: backend]].
			result isEmpty ifTrue:
				[result := NULL]].
		result == NULL ifTrue: [resTable := NULL] ifFalse: [resTable := result asImmuTable].
		newLocalIDs := myLocalIDs.
		id fetchBackend == NULL
			ifTrue: [newLocalIDs := (newLocalIDs withInt: id number) cast: IntegerRegion].
		^IDRegion make: mySpace
			with: newLocalIDs
			with: resTable
			with: myIncludesRest].
	^NULL "fodder"! !

!IDRegion methodsFor: 'protected: create'!

create: space {IDSpace | NULL}
	with: localIDs {IntegerRegion}
	with: importedIDs {ImmuTable | NULL of: Sequence and: IntegerRegion}
	with: includesRest {BooleanVar}
	
	super create.
	mySpace := space.
	myLocalIDs := localIDs.
	myImportedIDs := importedIDs.
	myIncludesRest := includesRest.! !

!IDRegion methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	| iDs {XnRegion} |
	oo << '{' << (self coordinateSpace cast: IDSpace) identifier << ' |'.
	iDs := myLocalIDs intersect: (IntegerRegion before: IntegerVarZero).
	iDs isEmpty ifFalse:
		[oo << ' !!' << iDs].
	iDs := myLocalIDs intersect: (IntegerRegion after: IntegerVarZero).
	iDs isEmpty ifFalse:
		[oo << ' ' << FeServer identifier << '.' << iDs].
	myImportedIDs == NULL ifFalse:
		[myImportedIDs stepper forPositions:
			[ :backend {Sequence} :iDs {IntegerRegion} |
			oo << ' ' << backend << '.' << iDs]].
	myIncludesRest ifTrue:
		[oo << ' ...{...} '].
	oo << '}'! !

!IDRegion methodsFor: 'smalltalk: passe'!

{SequenceRegion} identifiers
	"The identifiers of all the IDs in the region.
	Note: This is only possible if the region is finite, and even then it takes up much more space than the IDRegion. Use with caution. Tentative protocol."
	
	self passe! !

!IDRegion methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	mySpace _ receiver receiveHeaper.
	myLocalIDs _ receiver receiveHeaper.
	myImportedIDs _ receiver receiveHeaper.
	myIncludesRest _ receiver receiveBooleanVar.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	xmtr sendHeaper: mySpace.
	xmtr sendHeaper: myLocalIDs.
	xmtr sendHeaper: myImportedIDs.
	xmtr sendBooleanVar: myIncludesRest.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IDRegion class
	instanceVariableNames: ''!

(IDRegion getOrMakeCxxClassDescription)
	friends:
'friend class IDSpace;
friend class IDSimpleStepper;
friend class IDStepper;
friend class IDUpOrder;
';
	attributes: ((Set new) add: #CONCRETE; add: #ON.CLIENT; add: #COPY; yourself)!

!IDRegion class methodsFor: 'private:'!

{IDRegion} using: space {IDSpace | NULL}
	with: localIDs {IntegerRegion}
	with: importedIDs {ImmuTable | NULL of: Sequence and: IntegerRegion}
	with: includesRest {BooleanVar}
	"For IDSpace constructor only. Space had better be NULL if it's the global space"
	
	^self create: space with: localIDs with: importedIDs with: includesRest.! !

!IDRegion class methodsFor: 'smalltalk: init'!

linkTimeNonInherited

	TheLocalNumbers := NULL.
	TheGlobalNumbers := NULL.! !

!IDRegion class methodsFor: 'smalltalk: passe'!

make

	self passe!

make.ID: iD {ID}
	
	self passe!

make.ScruSet: iDs {ScruSet of: ID}
	self passe.
	^IDRegion create: false with: iDs asImmuSet! !

!IDRegion class methodsFor: 'smalltalk: initialization'!

initTimeNonInherited

	self REQUIRES: PrimSpec.
	self REQUIRES: IntegerRegion.
	TheLocalNumbers := IntegerRegion after: IntegerVarZero.
	TheGlobalNumbers := IntegerRegion before: IntegerVarZero.! !

!IDRegion class methodsFor: 'smalltalk: system'!

info.stProtocol
"{UInt8Array CLIENT} export
"! !

!IDRegion class methodsFor: 'creation'!

{IDRegion CLIENT} import: data {PrimIntArray}
	"Essential. Take some information describing an IDRegion and create the IDRegion it was exported from."
	
	| rcvr {SpecialistRcvr} spaceBackend {Sequence} spaceNumber {IntegerVar}
	  space {IDSpace} localIDs {IntegerRegion} n {IntegerVar} imported {ImmuTable | NULL}
	  includesRest {BooleanVar} |
	rcvr := Binary2XcvrMaker make
		makeRcvr: (TransferSpecialist make: Cookbook make)
		with: (XnReadStream make: (data cast: UInt8Array)).
	spaceBackend := ID importSequence: rcvr.
	spaceNumber := rcvr receiveIntegerVar.
	space := IDSpace make: spaceBackend with: spaceNumber.
	(space isEqual: CurrentGrandMap fluidGet globalIDSpace) ifTrue:
		[space := NULL].
	localIDs := ID importIntegerRegion: rcvr.
	n := rcvr receiveIntegerVar.
	n == IntegerVarZero ifTrue:
		[imported := NULL]
	ifFalse:
		[ | table {MuTable} |
		table := MuTable make: SequenceSpace make.
		1 to: n do: [ :i {IntegerVar} | | key {Sequence} value {IntegerRegion} |
			key := ID importSequence: rcvr.
			value := ID importIntegerRegion: rcvr.
			table at: key introduce: value].
		imported := table asImmuTable].
	includesRest := rcvr receiveIntegerVar DOTasLong.
	^self using: space with: localIDs with: imported with: includesRest!

make: space {IDSpace | NULL}
	with: localIDs {IntegerRegion}
	with: importedIDs {ImmuTable | NULL of: Sequence and: IntegerRegion}
	with: includesRest {BooleanVar}

	^ self create: space with: localIDs with: importedIDs with: includesRest! !