'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:03:07 pm'!

Heaper subclass: #PrimArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!
PrimArray comment:
'Array objects in smalltalk vary in size, while in x++ they allocate separate storage for the varying part.  Thus they require very different Recipes.

Recipes are normally registered by the class'' associated Recipe class at its own initialization time.  In smalltalk, the Array classes share a common Recipe class, (STArrayRecipe), which registers an instance of itself for each appropriate concrete subclass of PrimArray.X.

UInt1Arrays are odd in smalltalk, and get normal recipes generated by the normal smalltalk stubble mechanism

(WeakPtrArrays are not in the browser because they currently have no clients and may disappear.)'!

(PrimArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; add: #EQ; yourself)!

!PrimArray methodsFor: 'accessing'!

{void CLIENT login} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."
	self subclassResponsibility!

{Int32 CLIENT login} count
	"How many elements the array can hold"
	
	^self basicSize!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."
	self subclassResponsibility!

{Heaper CLIENT login} getValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."
	
	
	| result {Heaper} |
	result _ self fetchValue: index.
	result == NULL ifTrue: [Heaper BLAST: #NotInTable].
	^result!

{PrimSpec CLIENT} spec
	"A description of the kinds of things which can be stored in this array"
	
	self subclassResponsibility!

{Heaper CLIENT} valueAt: index {Int32}
	"Get an individual value from the array. It may be a primitive data type, or a pointer to an object. Blasts if it is NULL."
	
	| result {Heaper} |
	Ravi knownBug. "primitive types stuff"
	result := self fetchValue: index.
	result == NULL ifTrue:
		[Heaper BLAST: #MustNotBeNull].
	^result! !

!PrimArray methodsFor: 'destruction'!

{void} destroy! !

!PrimArray methodsFor: 'testing'!

{BooleanVar} contentsEqual: other {PrimArray}
	"Whether the two ranges contain semantically the same values. Two non-NULL pointers match iff the Heapers they point to are isEqual. Two integers match iff they have the same value, even though they may be represented as different sizes. Two floats likewise."
	
	self subclassResponsibility!

{UInt32} contentsHash
	"A hash of the entire contents of the array. If two arrays are contentsEqual, they will have the same contentsHash."

	^self elementsHash! !

!PrimArray methodsFor: 'bulk accessing'!

{void CLIENT login} at: to {Int32}
	storeMany: source {PrimArray}
	with: count {Int32 default: -1}
	with: from {Int32 default: Int32Zero}
	"Copy n elements from the source array into this one. The source array must be of a compatible type."

	|  delta {Int32} stop |
	delta _ to - from.
	count == -1 ifTrue: [stop _ source count] ifFalse: [stop _ from + count].
	(stop > source count or: [stop + delta > self count]) 
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	(self == source and: [from < to])
		ifTrue: [stop to: from+1 by: -1 do: [ :i | self basicAt: i + delta put: (source basicAt: i)]]
		ifFalse: [from+1 to: stop do: [ :i | self basicAt: i + delta put: (source basicAt: i)]]!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	with: before {Int32 default: Int32Zero}
	with: after {Int32 default: Int32Zero}
	"Make a copy of a piece of this array.  'Count' defaults to the count of the array minus 'start'.  'start' defaults to the beginning of the array, so with arguments, copy just copies the entire array.  'Before' and 'after' both default to 0.  They describe the count of nulls/0s to be added in the result array before the copied elements or after the copied elements.  For example, to copy everything after 10 into a new array leaving room for 20 elements, the incantation would be:
		array->copy(-1,10,20)"
	
	| copyCount {Int32} |
	count < Int32Zero
		ifTrue: [copyCount _ self count - start]
		ifFalse:
			[copyCount _ count.
			start + copyCount > self count
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]].
	^self class create: copyCount + before + after with: self with: start with: copyCount with: before!

{void CLIENT} copyFromBuffer: buffer {void star}
	with: start {Int32 default: Int32Zero}
	with: count {Int32 default: -1}

	[self knownBug. "doesn't check size and does the wrong thing in some cases for defautl count"
	self at: start storeElements: buffer with: Int32Zero with: count] smalltalkOnly.
	[Eric shouldImplement] translateOnly!

{PrimArray} copyGrow: after {Int32}
	"Make a copy of the array into a larger array.  The array has 'after' slots after the copied elements."
	
	^self copy: self count with: Int32Zero with: Int32Zero with: after!

{void} copyToBuffer: buffer {void star} with: size {Int32} with: count {Int32 default: -1} with: start {Int32 default: Int32Zero}

	[self knownBug. "doesn't check size"
	buffer at: Int32Zero storeMany: self with: count with: start] smalltalkOnly.
	[Eric shouldImplement] translateOnly!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self subclassResponsibility!

{Int32} indexOfElements: other {PrimArray}
	with: valueCount {Int32 default: -1}
	with: valueStart {Int32 default: Int32Zero}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of the given sequence of values at or after (before if n is negative) the given starting index, or -1 if there is none. Negative numbers for start are relative to the end of the array."
	
	| valueN result n forward |
	valueCount < 0
		ifTrue: [valueN := other basicSize]
		ifFalse: [valueCount > other basicSize
				ifTrue: [Heaper BLAST: #IndexOutOfBounds].
			valueN := valueCount].
	start >= 0
		ifTrue: [result := start]
		ifFalse: [result := self basicSize + start].
	n := nth abs.
	forward := nth > 0.
		[(forward ifTrue: [result > self basicSize - valueN] ifFalse: [result < 0]) ifTrue:
			[^-1].
		(self elementsEqual: result with: other with: valueStart with: valueN) ifTrue:
			[n := n - 1.
			n = 0 ifTrue:
				[^result]].
		forward
			ifTrue: [result := result + 1]
			ifFalse: [result := result - 1]] repeat!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self unimplemented!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	"Set a range of elements to have the same value"

	self subclassResponsibility! !

!PrimArray methodsFor: 'bulk testing'!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	with: n {Int32 default: -1}
	"Whether the two ranges contain the same values, using the criteria defined in contentsEqual"
	
	self subclassResponsibility!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero} 
	"A hash of the range of values out of the array. If two ranges are elementsEqual, they will have the same hash. For data values, additional zeros on the end make no difference to the hash."
	
	self subclassResponsibility! !

!PrimArray methodsFor: 'smalltalk: defaults'!

{void CLIENT login} at: to {Int32}
	storeMany: other {PrimArray}
	"Copy n elements from the other array into this one. The other array must be of a compatible type."
	
	^self at: to storeMany: other with: -1 with: 0!

{void CLIENT login} at: to {Int32}
	storeMany: other {PrimArray}
	with: count {Int32 default: -1}
	"Copy n elements from the other array into this one. The other array must be of a compatible type."
	
	^self at: to storeMany: other with: count with: 0!

{PrimArray CLIENT login} copy
	
	^self copy: -1 with: 0 with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	
	^self copy: count with: 0 with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	
	^self copy: count with: start with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	with: before {Int32 default: Int32Zero}
	
	^self copy: count with: start with: before with: 0!

{void} copyToBuffer: buffer {void start} with: size {Int32}
	self copyToBuffer: buffer with: size with: -1 with: Int32Zero!

{void} copyToBuffer: buffer {void start} with: size {Int32} with: count {Int32 default: -1}
	self copyToBuffer: buffer with: size with: count with: 0!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	
	^self elementsEqual: here with: other with: 0 with: -1!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	
	^self elementsEqual: here with: other with: there with: -1!

{UInt32} elementsHash
	^self elementsHash: -1 with: 0!

{UInt32} elementsHash: count {Int32 default: -1}
	^self elementsHash: count with: 0!

{Int32} indexOf: value {Heaper}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self indexOf: value with: 0 with: 1!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self indexOf: value with: start with: 1!

indexOfElements: other
	^self indexOfElements: other with: -1 with: 0 with: 0 with: 1!

indexOfElements: other with: valueCount
	^self indexOfElements: other with: valueCount with: 0 with: 0 with: 1!

indexOfElements: other with: valueCount with: valueStart 
	^self indexOfElements: other with: valueCount with: valueStart with: 0 with: 1!

indexOfElements: other with: valueCount with: valueStart with: start
	^self indexOfElements: other with: valueCount with: valueStart with: start with: 1!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	^ self indexPast: value with: start with: 1!

{void CLIENT login} storeAll
	^self storeAll: NULL with: -1 with: 0!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	^self storeAll: value with: -1 with: 0!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	^self storeAll: value with: count with: 0! !

!PrimArray methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	| before |
	before := '['.
	self count = 0 ifTrue:
		[oo << '[empty']
	ifFalse:
		[1 to: self basicSize do: [ :i {UInt32} |
			oo << before << (self basicAt: i).
			before := ' ']].
	oo << ']'! !

!PrimArray methodsFor: 'smalltalk: protected: implementation'!

{Int32} indexOfObject: value {IntegerVar | IEEE64 | Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	start < 0
		ifTrue: [result := self basicSize + start + 1.
			result < 0
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]]
		ifFalse: [result := start + 1].
	result > self basicSize
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth >= 0 ifTrue:
		[n := nth.
			[value = (self basicAt: result) ifTrue:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result < self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[value = (self basicAt: result) ifTrue:
				[n := n + 1.
				n == 0
					ifTrue: [^result]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]!

{Int32} indexPastObject: value {IntegerVar | IEEE64 | Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	(self basicSize = 0 or: [nth = 0]) ifTrue: [^-1].
	start < 0
		ifTrue: [result := self basicSize + start + 1]
		ifFalse: [result := start + 1].
	(result <= 0 or: [result > self basicSize])
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth > 0 ifTrue:
		[n := nth.
			[value = (self basicAt: result) ifFalse:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result <= self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[value = (self basicAt: result) ifFalse:
				[n := n + 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]! !

!PrimArray methodsFor: 'create'!

create: count {Int32} with: buffer {void star}
	"The buffer had better just behave like a Smalltalk array"

	super create.
	(buffer class == self class or: [buffer isKindOf: ByteString]) assert: 'Mismatched buffer types'.
	self at: Int32Zero storeMany: buffer! !

!PrimArray methodsFor: 'smalltalk: c arrays'!

at: i
	"act like a C array"
	
	^self basicAt: i + 1!

at: i put: value
	"act like a C array"
	
	self basicAt: i + 1 put: value.
	^value!

basicAt: index 
	"Answer the value of an indexable field in the receiver. Fail if the 
	argument index is not an Integer or is out of bounds. Essential. See 
	documentation in Object metaclass.  Do not override this message in any 
	subclass."

	<primitive: 60>
	index isInteger ifTrue: [^Heaper BLAST: #NotInTable].
	index respondsToArithmetic
		ifTrue: [^self basicAt: index asSmallInteger]
		ifFalse: [^self nonIntegerIndexError: index]!

basicAt: index put: value 
	"Store the second argument value in the indexable field of the receiver 
	indicated by index. Fail if the index is not an Integer or is out of bounds. Or 
	fail if the value is not of the right type for this kind of collection. Answer 
	the value that was stored. Essential. See documentation in Object 
	metaclass. Do not override in a subclass."

	<primitive: 61>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self basicSize])
					ifTrue: [^self improperStoreError]
					ifFalse: [^Heaper BLAST: #NotInTable]].
	index respondsToArithmetic
		ifTrue: [^self basicAt: index asSmallInteger put: value]
		ifFalse: [^self nonIntegerIndexError: index]! !

!PrimArray methodsFor: 'smalltalk: gc'!

{void} markInstances: count {IntegerVar} 
	"Don't do anything since most subclasses are bits.  Subclasses reimplement this."! !

!PrimArray methodsFor: 'generated:'!

actualHashForEqual ^self asOop!

isEqual: other ^self == other! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PrimArray class
	instanceVariableNames: ''!

(PrimArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; add: #EQ; yourself)!

!PrimArray class methodsFor: 'smalltalk: creation'!

create: len
	^ (self new: len) create: len!

create: len with: other
	"myAllocations _ myAllocations + 1."
	^ (self new: len) create: len with: other!

create: len with: other with: a with: b with: c
	"myAllocations _ myAllocations + 1."
	^ (self new: len) create: len with: other with: a with: b with: c! !