'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:08:27 pm'!

Abraham subclass: #TrailBlazer
	instanceVariableNames: '
		myTrail {BeEdition}
		myRecorded {HashSetCache of: BeRangeElement}
		myRefCount {IntegerVar}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-tclude'!
TrailBlazer comment:
'The object responsible for recording results into a trail. '!

(TrailBlazer getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #SHEPHERD.PATRIARCH; add: #COPY; add: #EQ; add: #LOCKED; add: #CONCRETE; yourself)!

!TrailBlazer methodsFor: 'create'!

create

	super create.
	myTrail := NULL.
	myRecorded := HashSetCache make.
	myRefCount := IntegerVarZero.
	self newShepherd.! !

!TrailBlazer methodsFor: 'private:'!

{void} setEdition: trail {BeEdition}

	myTrail := trail.
	self diskUpdate.! !

!TrailBlazer methodsFor: 'accessing'!

{BooleanVar} isAlive
	"Whether this TrailBlazer was in fact successfully attached"
	
	^myTrail ~~ NULL!

{void} record: answer {BeRangeElement}
	"record the answer into my Edition, and keep only the partial part.

	Should usually suppress redundant records of the same object.  (These are typically generated by a race between the now and future parts of a backfollow, which are guaranteed to err by overlapping rather than gapping.  They may also be generated by a crash/reboot during AgendaItem processing.)"

	(myRecorded hasMember: answer) ifFalse:
		[ | iD {ID} newTrail {BeEdition} |
		iD := (myTrail coordinateSpace cast: IDSpace) newID.
		TrailBlazer problems.RecordFailure
			handle: [ :ex | ^VOID]
			do: [(myTrail get: iD) makeIdentical: (answer makeFe: NULL)].
		myRecorded store: answer.
		Ravi thingToDo.  "This should not be an edit operation (?)"
		newTrail := myTrail without: iD.
		Ravi thingToDo. "decrease refcount on old trail, increase on new one"
		DiskManager consistent: 1 with:
			[myTrail := newTrail.
			self diskUpdate]]! !

!TrailBlazer methodsFor: 'storage'!

{void} addReference: object {Abraham unused} 
	"Increment the reference count"

	DiskManager consistent: 1
		with: 
			[myRefCount := myRefCount + 1.
			myRefCount = 1 ifTrue: [self remember].
			self diskUpdate]!

{void} removeReference: object {Abraham unused} 
	"Decrement the reference count"

	DiskManager consistent: 1
		with: 
			[myRefCount := myRefCount - 1.
			myRefCount = IntegerVarZero ifTrue: [self forget].
			self diskUpdate]! !

!TrailBlazer methodsFor: 'generated:'!

actualHashForEqual ^self asOop!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	myTrail _ receiver receiveHeaper.
	myRecorded _ receiver receiveHeaper.
	myRefCount _ receiver receiveIntegerVar.!

isEqual: other ^self == other!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	xmtr sendHeaper: myTrail.
	xmtr sendHeaper: myRecorded.
	xmtr sendIntegerVar: myRefCount.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TrailBlazer class
	instanceVariableNames: ''!

(TrailBlazer getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #SHEPHERD.PATRIARCH; add: #COPY; add: #EQ; add: #LOCKED; add: #CONCRETE; yourself)!

!TrailBlazer class methodsFor: 'create'!

make: trail {BeEdition} 
	"should only be called from Edition::getOrMakeTrailBlazer"

	| result {TrailBlazer} partial {XnRegion} sub {BeEdition} |
	DiskManager consistent: 1 with:
		[result := self create].
	partial := trail attachTrailBlazer: result.
	sub := trail copy: partial.
	DiskManager consistent: 1 with:
		[result setEdition: sub]. "this makes the blazer be alive, once attached"
	^result! !

!TrailBlazer class methodsFor: 'exceptions:'!

problems.RecordFailure

	^Heaper signals: #(MustBeOwner CantMakeIdentical NotInTable)! !