'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:02:32 pm'!

XuCoordinateSpace subclass: #XuIDSpace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-Spaces-Unordered'!
XuIDSpace comment:
'A space of IDs, which can generate globally unique IDs.

Implementation note:
	myBackend - the identifier of the Server which generated this space. If NULL, then it was generated by the current Server (unless mySpaceNumber is -1, in which case it is the single global IDSpace shared by all Servers.
	mySpaceNumber - identifies which space this is. If -1, then it is the global ID space, and myBackend must be NULL.'!

(XuIDSpace getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; yourself)!

!XuIDSpace methodsFor: 'promise access'!

{IDSpace} getActualIDSpace
	self isBroken
		ifFalse: [^super actualThing cast: IDSpace.]
		ifTrue: [Heaper BLAST: #BrokenPromise].! !

!XuIDSpace methodsFor: 'creation'!

create.Problem: problem {Problem}
	super create.Problem: problem.!

create.Promise: promise {XuPromise}
	super create.Promise: promise.!

create: anIDSpace {IDSpace}
	super create: anIDSpace.! !

!XuIDSpace methodsFor: 'accessing'!

{XuIntArray} export

	"Essential. Produce an array which can be handed to Server::importIDSpace on any Server to get back the same IDSpace"
	
	| retVal {XuIntArray} |
	(self isBroken) ifTrue:
		[^XuIntArray makeBrokenBy: self].

	Heaper problems.AllBlasts 
		handle: [:ex | retVal _ XuIntArray makeBroken: ex asProblem. ex return]
		do: 	[retVal _ XuIntArray make: (self getActualIDSpace export)].
	XuPromise inDelayBlock ifFalse:
		[XuPromise force.
		retVal isBroken ifTrue: [retVal explode]].
	^retVal! !

!XuIDSpace methodsFor: 'making'!

{XuIDRegion} iDsFromServer: identifier {XuSequence} 

	"Essential. The Region of IDs in this space which might be genrated by the given Server"
	
	| retVal {XuIDRegion} |
	(self isBroken) ifTrue:
		[^XuIDRegion makeBrokenBy: self].
	(identifier isBroken) ifTrue:
		[^XuIDRegion makeBrokenBy: identifier].

	Heaper problems.AllBlasts 
		handle: [:ex | retVal _ XuIDRegion makeBroken: ex asProblem. ex return]
		do: 	[retVal _ XuIDRegion make: (self getActualIDSpace iDsFromServer: ((XuPromise getActual: identifier) cast: Sequence))].
	XuPromise inDelayBlock ifFalse:
		[XuPromise force.
		retVal isBroken ifTrue: [retVal explode]].
	^retVal!

{XuID} newID

	"Essential. A new ID guaranteed to be different from every other newID generated by this IDSpace or any IDSpace isEqual to it, on any Server. (Although of course IDs generated using this->oldID () may conflict if the right numbers happen to have been supplied.)"
	
	| retVal {XuID} |
	(self isBroken) ifTrue:
		[^XuID makeBrokenBy: self].

	Heaper problems.AllBlasts 
		handle: [:ex | retVal _ XuID makeBroken: ex asProblem. ex return]
		do: 	[retVal _ XuID make: (self getActualIDSpace newID)].
	XuPromise inDelayBlock ifFalse:
		[XuPromise force.
		retVal isBroken ifTrue: [retVal explode]].
	^retVal!

{XuIDRegion} newIDs: count {XuIntValue} 

	"A region containing a finite number of globally unique IDs. See newID for uniqueness guarantees."
	
	| retVal {XuIDRegion} |
	(self isBroken) ifTrue:
		[^XuIDRegion makeBrokenBy: self].
	(count isBroken) ifTrue:
		[^XuIDRegion makeBrokenBy: count].

	Heaper problems.AllBlasts 
		handle: [:ex | retVal _ XuIDRegion makeBroken: ex asProblem. ex return]
		do: 	[retVal _ XuIDRegion make: (self getActualIDSpace newIDs:count getActualIntegerVar)].
	XuPromise inDelayBlock ifFalse:
		[XuPromise force.
		retVal isBroken ifTrue: [retVal explode]].
	^retVal! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

XuIDSpace class
	instanceVariableNames: ''!

(XuIDSpace getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; yourself)!

!XuIDSpace class methodsFor: 'pseudo-constructors'!

make: anIDSpace {IDSpace}
	^self create: anIDSpace!

{XuIDSpace} makeBroken: problem {Problem star}
	| broke {XuIDSpace} |
	broke _ self create.Problem: problem.
	XuPromise savePromise: broke.
	^broke!

{XuIDSpace} makeBrokenBy: broken {XuPromise}
	^self create.Promise: broken.! !