/* Copyright Xanadu Operating Company.  All Rights Reserved. */

/******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Xanadu    *
* Operating Company, and considered a trade secret as defined in section     * 
* 499C of the penal code of the State of California.  Use of this information* 
* by anyone other than authorized employees of Xanadu is granted             *
* only under a  written non-disclosure agreement, expressly prescribing      * 
* the scope and  manner of such use.                                         *
*                                                                            *
***************************************************************************
Output from Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989
*/

#ifndef IDX_HXX
#define IDX_HXX


#ifndef TOFUX_HXX
#include "tofux.hxx"
#endif /* TOFUX_HXX */

#ifndef INTVARX_HXX
#include "intvarx.hxx"
#endif /* INTVARX_HXX */

#ifndef IDX_OXX
#include "idx.oxx"
#endif /* IDX_OXX */


#ifndef INITX_HXX
#include "initx.hxx"
#endif /* INITX_HXX */

#ifndef SPACER_HXX
#include "spacer.hxx"
#endif /* SPACER_HXX */

#ifndef SPACEX_HXX
#include "spacex.hxx"
#endif /* SPACEX_HXX */


#ifndef COUNTERX_OXX
#include "counterx.oxx"
#endif /* COUNTERX_OXX */

#ifndef INTEGERX_OXX
#include "integerx.oxx"
#endif /* INTEGERX_OXX */

#ifndef NXCVRX_OXX
#include "nxcvrx.oxx"
#endif /* NXCVRX_OXX */

#ifndef PARRAYX_OXX
#include "parrayx.oxx"
#endif /* PARRAYX_OXX */

#ifndef PRIMVALX_OXX
#include "primvalx.oxx"
#endif /* PRIMVALX_OXX */

#ifndef SEQUENCX_OXX
#include "sequencx.oxx"
#endif /* SEQUENCX_OXX */

#ifndef SETX_OXX
#include "setx.oxx"
#endif /* SETX_OXX */

#ifndef STEPPERX_OXX
#include "stepperx.oxx"
#endif /* STEPPERX_OXX */

#ifndef TABLESX_OXX
#include "tablesx.oxx"
#endif /* TABLESX_OXX */

#ifndef XFRSPECX_OXX
#include "xfrspecx.oxx"
#endif /* XFRSPECX_OXX */


/*  */
/*  */




/* ************************************************************************ *
 * 
 *                    Class ID 
 *
 * ************************************************************************ */




	/* Implementation note:
	
	An ID exists within a particular IDSpace, and is generated by 
	a particular Server. It holds onto the space and the Server 
	which created it, along with a number identifying the ID 
	uniquely. If mySpace is NULL, then the ID is in the global 
	IDSpace. If myBackend is NULL, then this ID was generated by 
	the current Server (unless myNumber is negative, in which 
	case it is considered to have been generated by the "global" 
	backend). If myBackend is non-NULL, then myNumber must be 
	non-negative. */

class ID : public Position {

/* Attributes for class ID */
	CONCRETE(ID)
	ON_CLIENT(ID)
	COPY(ID,DiskCuisine)
	AUTO_GC(ID)
  public: /* module private create */

	
	static RPTR(ID) make (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(Sequence) OR(NULL) ARG(backend), 
			IntegerVar ARG(number))
	;
	
  private: /* private: pseudo constructors */

	/* Special for IDStepper - checks whether it should make 
	backend be NULL */
	
	static RPTR(ID) usingx (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(Sequence) OR(NULL) ARG(backend), 
			IntegerVar ARG(number))
	;
	
  public: /* creation */

	/* Essential. Take some information describing an ID and 
	create the ID it was exported from. */
	
	static CLIENT RPTR(ID) import (APTR(PrimIntArray) ARG(data));
	
  private: /* private: export/import for friends */

	/* Write a IntegerRegion onto a stream */
	
	static void exportIntegerRegion (APTR(SpecialistXmtr) ARG(xmtr), APTR(IntegerRegion) ARG(integers));
	
	/* Write a Sequence onto a stream */
	
	static void exportSequence (APTR(SpecialistXmtr) ARG(xmtr), APTR(Sequence) ARG(sequence));
	
	/* Read a IntegerRegion from a stream */
	
	static RPTR(IntegerRegion) importIntegerRegion (APTR(SpecialistRcvr) ARG(rcvr));
	
	/* Read a Sequence from a stream */
	
	static RPTR(Sequence) importSequence (APTR(SpecialistRcvr) ARG(rcvr));
	
  public: /* accessing */

	
	virtual RPTR(XnRegion) asRegion ();
	
	
	virtual RPTR(CoordinateSpace) coordinateSpace ();
	
	/* Essential. Export this iD in a form which can be handed to 
	Server::importID on any Server to generate the same ID */
	
	virtual CLIENT RPTR(UInt8Array) export ();
	
  public: /* comparing */

	
	virtual UInt32 actualHashForEqual ();
	
	
	virtual BooleanVar isEqual (APTR(Heaper) ARG(heaper));
	
  protected: /* protected: create */

	
	ID (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(Sequence) OR(NULL) ARG(backend), 
			IntegerVar ARG(number))
	;
	
  public: /* printing */

	
	virtual void printOn (ostream& ARG(oo));
	
  private: /* private: */

	/* Essential. A Sequence identifying the server on which this 
	was created */
	
	virtual RPTR(Sequence) backend ();
	
	
	virtual RPTR(Sequence) OR(NULL) fetchBackend ();
	
	
	virtual RPTR(IDSpace) OR(NULL) fetchSpace ();
	
	/* Essential. The number identifying this ID from all others 
	generated by the same Server in the same IDSpace. */
	
	virtual IntegerVar number ();
	
  public: /* obsolete: */

	/* A sequence of numbers which uniquely identify this ID 
	within its space */
	
	virtual RPTR(Sequence) identifier ();
	
  private:
	CHKPTR(IDSpace) OR(NULL) mySpace;
	CHKPTR(Sequence) OR(NULL) myBackend;
	IntegerVar myNumber;
/* Friends for class ID */
friend class IDRegion;
friend class IDStepper;
friend class IDUpOrder;
friend class IDTester;
friend class IDSpace;



};  /* end class ID */



/* ************************************************************************ *
 * 
 *                    Class IDDsp 
 *
 * ************************************************************************ */



/* Initializers for IDDsp */
/* Declaration inherited from IdentityDsp */




/* Declaration inherited from IdentityDsp */





	/* There are no non-trivial Dsps on IDs. */

class IDDsp : public IdentityDsp {

/* Attributes for class IDDsp */
	CONCRETE(IDDsp)
	PSEUDO_COPY(IDDsp,DiskCuisine)
	AUTO_GC(IDDsp)

/* Initializers for IDDsp */
/* Declaration inherited from IdentityDsp */




/* Declaration inherited from IdentityDsp */

friend class INIT_TIME_NAME(IDDsp,initTimeInherited);

  public: /* rcvr pseudo constructors */

	
	static RPTR(Heaper) make (APTR(Rcvr) ARG(rcvr));
	
  public: /* pseudo constructors */

	
	static RPTR(IDDsp) make (APTR(IDSpace) ARG(space));
	
  public: /* accessing */

	
	virtual RPTR(CoordinateSpace) coordinateSpace ();
	
  public: /* creation */

	
	IDDsp ();
	
	
	IDDsp (APTR(IDSpace) ARG(space), TCSJ);
	
  private:
	CHKPTR(IDSpace) mySpace;

  /* ---------- Static Member variables (class inst vars) ----------- */
  private:
	static IdentityDsp  * theDsp;
};  /* end class IDDsp */



/* ************************************************************************ *
 * 
 *                    Class IDRegion 
 *
 * ************************************************************************ */



/* Initializers for IDRegion */







	/* If mySpace is NULL, then it is assumed to be global IDSpace.
	The non-negative part of myLocalIDs contains the number 
	portion of all IDs in the region that were generated by the 
	current backend. The negative part contains all IDs which 
	were generated by the "global" backend.
	If myImportedIDs is NULL, it is considered to be empty. If it 
	is non-NULL, it must be non-empty. It contains the numbers of 
	all IDs in this region which were generated by other 
	backends. If an entry in the table could be omitted without 
	effect, then it should be omitted. (i.e. if myIncludesRest 
	and the region is full, or myIncludesRest not and the region 
	is empty, then the entry should be omitted).
	If myIncludesRest is true, then the region includes, in 
	addition to those IDs explictly listed in myLocalIDs and 
	myImportedIDs, all ID generated by all other backends. */

class IDRegion : public XnRegion {

/* Attributes for class IDRegion */
	CONCRETE(IDRegion)
	ON_CLIENT(IDRegion)
	COPY(IDRegion,DiskCuisine)
	AUTO_GC(IDRegion)

/* Initializers for IDRegion */



friend class INIT_TIME_NAME(IDRegion,initTimeNonInherited);

  private: /* private: */

	/* For IDSpace constructor only. Space had better be NULL if 
	it's the global space */
	
	static RPTR(IDRegion) usingx (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(IntegerRegion) ARG(localIDs), 
			APTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) ARG(importedIDs), 
			BooleanVar ARG(includesRest))
	;
	
  public: /* creation */

	/* Essential. Take some information describing an IDRegion 
	and create the IDRegion it was exported from. */
	
	static CLIENT RPTR(IDRegion) import (APTR(PrimIntArray) ARG(data));
	
	
	static RPTR(IDRegion) make (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(IntegerRegion) ARG(localIDs), 
			APTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) ARG(importedIDs), 
			BooleanVar ARG(includesRest))
	;
	
  public: /* accessing */

	
	virtual RPTR(XnRegion) asSimpleRegion ();
	
	
	virtual RPTR(CoordinateSpace) coordinateSpace ();
	
	/* Essential. Export the IDRegion in a form that can be used 
	to recreate it with IDRegion::import. */
	
	virtual CLIENT RPTR(UInt8Array) export ();
	
	
	virtual RPTR(Position) theOne ();
	
  public: /* testing */

	
	virtual UInt32 actualHashForEqual ();
	
	
	virtual BooleanVar hasMember (APTR(Position) ARG(position));
	
	
	virtual BooleanVar isEmpty ();
	
	
	virtual BooleanVar isEqual (APTR(Heaper) ARG(other));
	
	
	virtual BooleanVar isFinite ();
	
	
	virtual BooleanVar isFull ();
	
	
	virtual BooleanVar isSimple ();
	
	
	virtual BooleanVar isSubsetOf (APTR(XnRegion) ARG(region));
	
  protected: /* protected: enumerating */

	
	virtual RPTR(Stepper) OF1(Position) actualStepper (APTR(OrderSpec) ARG(order) = NULL);
	
  public: /* enumerating */

	
	virtual IntegerVar count ();
	
	
	virtual RPTR(ScruSet) OF1(XnRegion) distinctions ();
	
	
	virtual RPTR(Stepper) simpleRegions (APTR(OrderSpec) ARG(order) = NULL);
	
  private: /* private: */

	/* All backends which have generated IDs in this Region */
	
	virtual RPTR(SequenceRegion) backends ();
	
	/* All backends which are non-empty and are explicitly 
	listed. For IDSimpleStepper */
	
	virtual RPTR(XnRegion) OF1(Sequence) explicitBackends ();
	
	
	virtual RPTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) fetchImportedIDs ();
	
	/* The region which covers material not in the 
	explicitBackends list, or NULL if there is none. */
	
	virtual RPTR(IDRegion) fetchInexplicit ();
	
	
	virtual RPTR(IDSpace) fetchSpace ();
	
	
	virtual RPTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) getImportedIDs ();
	
	/* The numbers of all IDs in this region that were generated 
	by the given backend */
	
	virtual RPTR(XnRegion) OF1(IntegerPos) iDNumbersFrom (APTR(Sequence) ARG(backend));
	
	
	virtual BooleanVar includesRest ();
	
	
	virtual RPTR(IntegerRegion) localIDs ();
	
  public: /* operations */

	
	virtual RPTR(XnRegion) complement ();
	
	
	virtual RPTR(XnRegion) intersect (APTR(XnRegion) ARG(region));
	
	
	virtual RPTR(XnRegion) simpleUnion (APTR(XnRegion) ARG(region));
	
	
	virtual RPTR(XnRegion) unionWith (APTR(XnRegion) ARG(region));
	
	
	virtual RPTR(XnRegion) with (APTR(Position) ARG(pos));
	
  protected: /* protected: create */

	
	IDRegion (
			APTR(IDSpace) OR(NULL) ARG(space), 
			APTR(IntegerRegion) ARG(localIDs), 
			APTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) ARG(importedIDs), 
			BooleanVar ARG(includesRest))
	;
	
  public: /* printing */

	
	virtual void printOn (ostream& ARG(oo));
	
  private:
	CHKPTR(IDSpace) OR(NULL) mySpace;
	CHKPTR(IntegerRegion) myLocalIDs;
	CHKPTR(ImmuTable) OF2(Sequence,IntegerRegion) OR(NULL) myImportedIDs;
	BooleanVar myIncludesRest;

  /* ---------- Static Member variables (class vars) ----------- */
  private:
	static GPTR(IntegerRegion) TheGlobalNumbers;
	static GPTR(IntegerRegion) TheLocalNumbers;
/* Friends for class IDRegion */
friend class IDSpace;
friend class IDSimpleStepper;
friend class IDStepper;
friend class IDUpOrder;



};  /* end class IDRegion */



/* ************************************************************************ *
 * 
 *                    Class IDSpace 
 *
 * ************************************************************************ */




	/* A space of IDs, which can generate globally unique IDs.
	
	Implementation note:
		myBackend - the identifier of the Server which generated 
	this space. If NULL, then it was generated by the current 
	Server (unless mySpaceNumber is -1, in which case it is the 
	single global IDSpace shared by all Servers.
		mySpaceNumber - identifies which space this is. If -1, then 
	it is the global ID space, and myBackend must be NULL. */

class IDSpace : public CoordinateSpace {

/* Attributes for class IDSpace */
	CONCRETE(IDSpace)
	PSEUDO_COPY(IDSpace,DiskCuisine)
	ON_CLIENT(IDSpace)
	AUTO_GC(IDSpace)
  public: /* creation */

	/* Return the global ID space. */
	
	static CLIENT RPTR(IDSpace) global ();
	
	/* Essential. Take some information describing an IDSpace and 
	create the IDSpace it was exported from. */
	
	static CLIENT RPTR(IDSpace) import (APTR(PrimIntArray) ARG(data));
	
	/* Essential. Create a new globally unique space of IDs */
	
	static CLIENT RPTR(IDSpace) unique ();
	
  private: /* private: pseudo constructors */

	
	static RPTR(IDSpace) make (APTR(Sequence) OR(NULL) ARG(identifier), IntegerVar ARG(number));
	
	
	static RPTR(IDSpace) make (
			APTR(Sequence) OR(NULL) ARG(identifier), 
			IntegerVar ARG(number), 
			APTR(Counter) ARG(counter))
	;
	
  public: /* rcvr pseudo constructors */

	
	static RPTR(Heaper) make (APTR(Rcvr) ARG(rcvr));
	
  public: /* making */

	/* Essential. The Region of IDs in this space which might be 
	genrated by the given Server */
	
	virtual CLIENT RPTR(IDRegion) iDsFromServer (APTR(Sequence) ARG(identifier));
	
	/* Essential. A new ID guaranteed to be different from every 
	other newID generated by this IDSpace or any IDSpace isEqual 
	to it, on any Server. (Although of course IDs generated using 
	this->oldID () may conflict if the right numbers happen to 
	have been supplied.) */
	
	virtual CLIENT RPTR(ID) newID ();
	
	/* A region containing a finite number of globally unique 
	IDs. See newID for uniqueness guarantees. */
	
	virtual CLIENT RPTR(IDRegion) newIDs (IntegerVar ARG(count));
	
  private: /* private: for friends */

	/* Essential. The Server which created this IDSpace */
	
	virtual RPTR(Sequence) backend ();
	
	
	virtual RPTR(Sequence) OR(NULL) fetchBackend ();
	
	/* NULL if this is the global IDSpace, self otherwise */
	
	virtual RPTR(IDSpace) OR(NULL) fetchIDSpace ();
	
	/* Recreate a region of IDs from information that was stored 
	outside the Server */
	
	virtual RPTR(IDRegion) oldIDs (APTR(Sequence) ARG(backend), APTR(IntegerRegion) ARG(numbers));
	
	/* Essential. Identifies this particular space among all 
	those generated by the same Server. */
	
	virtual IntegerVar spaceNumber ();
	
  private: /* private: create */

	
	IDSpace (
			APTR(Sequence) OR(NULL) ARG(backend), 
			IntegerVar ARG(number), 
			APTR(Counter) ARG(counter))
	;
	
	
	virtual void finishCreation ();
	
  public: /* testing */

	
	virtual UInt32 actualHashForEqual ();
	
	
	virtual BooleanVar isEqual (APTR(Heaper) ARG(other));
	
  public: /* printing */

	
	virtual void printOn (ostream& ARG(oo));
	
  public: /* accessing */

	/* Essential. Produce an array which can be handed to 
	Server::importIDSpace on any Server to get back the same IDSpace */
	
	virtual CLIENT RPTR(UInt8Array) export ();
	
  public: /* obsolete: */

	/* A Sequence uniquely identifying this IDSpace, so that
			FeServer::current ()->oldIDSpace (this->identifier ())
				->isEqual (this) */
	
	virtual RPTR(Sequence) identifier ();
	
  public: /* hooks: */

	
	virtual SEND_HOOK void sendIDSpaceTo (APTR(Xmtr) ARG(xmtr));
	
  private:
	CHKPTR(Sequence) OR(NULL) myBackend;
	IntegerVar mySpaceNumber;
	CHKPTR(Counter) myNewIDCounter;
/* Friends for class IDSpace */
friend IDSimpleStepper;
friend class BeGrandMap;
friend class IDTester;
friend class ID;
friend class IDRegion;


};  /* end class IDSpace */



#endif /* IDX_HXX */

