/* Copyright Xanadu Operating Company.  All Rights Reserved. */

/******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Xanadu    *
* Operating Company, and considered a trade secret as defined in section     * 
* 499C of the penal code of the State of California.  Use of this information* 
* by anyone other than authorized employees of Xanadu is granted             *
* only under a  written non-disclosure agreement, expressly prescribing      * 
* the scope and  manner of such use.                                         *
*                                                                            *
***************************************************************************
Output from Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989
*/

#ifndef TRACEPX_HXX
#define TRACEPX_HXX


#ifndef TOFUX_HXX
#include "tofux.hxx"
#endif /* TOFUX_HXX */

#ifndef INTVARX_HXX
#include "intvarx.hxx"
#endif /* INTVARX_HXX */

#ifndef TRACEPX_OXX
#include "tracepx.oxx"
#endif /* TRACEPX_OXX */


#ifndef INITX_HXX
#include "initx.hxx"
#endif /* INITX_HXX */


#ifndef BRANCHX_OXX
#include "branchx.oxx"
#endif /* BRANCHX_OXX */

#ifndef PRIMTABX_OXX
#include "primtabx.oxx"
#endif /* PRIMTABX_OXX */

#ifndef SETX_OXX
#include "setx.oxx"
#endif /* SETX_OXX */


/*  */
/*  */




/* ************************************************************************ *
 * 
 *                    Class TracePosition 
 *
 * ************************************************************************ */




	/* NO CLASS COMMENT */

class TracePosition : public Heaper {

/* Attributes for class TracePosition */
	DEFERRED(TracePosition)
	NO_GC(TracePosition)
  public: /* pseudo-constructors */

	
	static RPTR(TracePosition) make (APTR(BranchDescription) ARG(branch), UInt32 ARG(position));
	
  public: /* testing */

	/* Hash based on both the branch and position. */
	
	virtual UInt32 actualHashForEqual () DEFERRED_FUNC;
	
	
	virtual BooleanVar isEqual (APTR(Heaper) ARG(another)) DEFERRED_FUNC;
	
	
	virtual BooleanVar isLE (APTR(TracePosition) ARG(another)) DEFERRED_FUNC;
	
  public: /* accessing */

	
	virtual RPTR(BranchDescription) branch () DEFERRED_FUNC;
	
	
	virtual void cacheIn (APTR(PrimIndexTable) ARG(navCache)) DEFERRED_SUBR;
	
	
	virtual RPTR(TracePosition) newSuccessor () DEFERRED_FUNC;
	
	/* Return a new tracePosition that is after both the receiver 
	and tracePos. */
	
	virtual RPTR(TracePosition) newSuccessorAfter (APTR(TracePosition) ARG(tracePos)) DEFERRED_FUNC;
	
	
	virtual UInt32 position () DEFERRED_FUNC;
	

	/* automatic 0-argument constructor */
  public:
	TracePosition();

};  /* end class TracePosition */



/* ************************************************************************ *
 * 
 *                    Class   BoundedTrace 
 *
 * ************************************************************************ */




	/* Trace Positions represent absolute coordinates in a 
	tree-ordered space.  
	The ent navigates to O-roots from H-roots in the northern 
	direction using 
	TracePositions.  Refer to the Ent document for a description 
	of the current 
	implementation.  TracePositions are generated by asking 
	another tracePosition
	for a newSuccesor.  The starting tracePosition is the initil 
	TraceTree.  It 
	corresponds to branch 1, position 1. */

class BoundedTrace : public TracePosition {

/* Attributes for class BoundedTrace */
	CONCRETE(BoundedTrace)
	COPY(BoundedTrace,DiskCuisine)
	AUTO_GC(BoundedTrace)
  public: /* testing */

	/* This isn't used now.  It might be later.  Hash 
		 based on both the branch and position  This hash
		 uses a couple of arbitrary primes. */
	
	virtual UInt32 actualHashForEqual ();
	
	
	virtual BooleanVar isEqual (APTR(Heaper) ARG(another));
	
	/* Return true if the two positions are comparable and the 
	receiver is  
		less than the argument. The argument is put in the cache so that  
		many comparisons IN THE SAME ORDER will occur very fast. This  
		could probably check for both tracePositions in the cache for speed  
		even when the arguments are swapped. */
	
	virtual BooleanVar isLE (APTR(TracePosition) ARG(another));
	
  public: /* accessing */

	/* This should only be accessible in this module. */
	
	virtual RPTR(BranchDescription) branch ();
	
	/* Cache the nav-data for the receiver in navCache. */
	
	virtual void cacheIn (APTR(PrimIndexTable) ARG(navCache));
	
	/* Return a new successor to the receiver. The first 
	successor is on the same 
		branch with a higher position. Further successors are alllocated in a 
		binary-tree fashion along a new branch. */
	
	virtual RPTR(TracePosition) newSuccessor ();
	
	/* Return a new tracePosition that is after both the receiver 
	and tracePos. */
	
	virtual RPTR(TracePosition) newSuccessorAfter (APTR(TracePosition) ARG(trace));
	
	/* This should only be accessible in this module. */
	
	virtual UInt32 position ();
	
	/* Return all the successors of the receiver. */
	
	virtual RPTR(ImmuSet) OF1(TracePosition) successors ();
	
  public: /* create */

	
	BoundedTrace (APTR(BranchDescription) ARG(br), UInt32 ARG(pos));
	
  private:
	CHKPTR(BranchDescription) myBranch;
	UInt32 myPosition;
	friend class TracePosition;
};  /* end class BoundedTrace */



#endif /* TRACEPX_HXX */

