/* ========================================================================== *
 *
 *	Copyright (c) 1989 by Xanadu Operating Company, All Rights Reserved.
 *
 * ========================================================================== * 
 *
 * The information contained herein is confidential, proprietary to Xanadu
 * Operating Company, and considered a trade secret as defined in section
 * 499C of the penal code of the State of California.
 *
 * Use of this information by anyone other than authorized employees of
 * Xanadu is granted only under a written nondisclosure agreement,
 * expressly prescribing the scope and manner of such use.
 *
 * The above copyright notice is not to be construed as evidence of
 * publication or the intent to publish.
 *
 * ========================================================================== * 
 *
 *				cbombc.h
 *
 *	C interface to the bailout bomb macros and their associated objects.
 *
 *	By Michael McClary		1989
 *
 * ========================================================================== * 
 *
 *	Repitition suppression switch
 *
 * ========================================================================== */

#ifndef CBOMBC_H
#define CBOMBC_H

/* $Id: cbombc.h,v 1.5 1992/11/25 23:26:02 eric Exp $ */

/* ========================================================================== *
 *
 *	Includes
 *
 * ========================================================================== */

#ifndef BOMBC_H
#include "bombc.h"
#endif /* BOMBC_H */

#ifndef	IN_CBOMBX_CXX

typedef	enum SourceOfDetonationSignal	SourceOfDetonationSignal;

#endif	/* IN_CBOMBX_CXX */

#include <setjmp.h>

/* ========================================================================== 
 *
 *	C_PROBLEM_LIST(LTAG,NUMBER,(LIST)):
 *
 * ========================================================================== */

#define	C_PROBLEM_LIST(LTAG, NUMBER, LIST)	PROBLEM_LIST(LTAG,NUMBER,LIST)

/* ==========================================================================
 *
 *	FIX THESE !!!!
 *
 * ==========================================================================
 *
 *	The defines need to be included from a file generated by something
 *	that includes the class statements and computes a sizeof().
 *
 * ========================================================================== */

#define	C_BOMB_SIZE	5
#ifdef sgi
#	define C_SHIELD_SIZE 34
#else /* sun */
#	define	C_SHIELD_SIZE	15
#endif /* sgi or sun */

#ifndef	C_BOMB_SIZE
#define	C_BOMB_SIZE	10
#endif	/* C_BOMB_SIZE */

#ifndef	C_SHIELD_SIZE
#define	C_SHIELD_SIZE	20
#endif	/* C_SHIELD_SIZE */

typedef	struct CBomb {
	UInt32		someStorage[C_BOMB_SIZE];
} CBomb;

typedef	struct CShield {
	UInt32		someStorage[C_SHIELD_SIZE];
} CShield;

/* ========================================================================== 
 *
 *	C_BLAST()
 *	C_BLAST_WITH_VAL()
 *	C_BLAST_FROM_THE_PASSED()
 *
 *	Detonate a string of bombs, until all blow or one catches the error.
 *
 * ========================================================================== */

extern void cBlast(
#ifdef	IN_CBOMBX_CXX
		  char *	argProblemName
		, int		argVal
#ifdef	BOMB_REPORT_LINE
		, char *	argFileName
		, int		argLineNumber
#endif	/* BOMB_REPORT_LINE */
#endif	/* IN_CBOMBX_CXX */
	);

extern void cReblast(
#ifdef	IN_CBOMBX_CXX
	  void *
#endif	/* IN_CBOMBX_CXX */
);

#define	C_BLAST(argProblemName)	C_BLAST_WITH_VAL(argProblemName,1)

#ifdef	BOMB_REPORT_LINE
#define	C_BLAST_WITH_VAL(argProblemName, val) {				\
		cBlast(STR(argProblemName), val, __FILE__, __LINE__);	\
	}

#define	C_BLAST_FROM_THE_PASSED(argProblemName, val, argFILE, argLINE) { \
		cBlast(STR(argProblemName), val, argFILE,  argLINE);	\
	}

#else /* BOMB_REPORT_LINE */
#define	C_BLAST_WITH_VAL(argProblemName, val) {				\
		cBlast(STR(argProblemName), val);			\
	}

#define	C_BLAST_FROM_THE_PASSED(argProblemName, val, argFILE, argLINE) { \
		cBlast(STR(argProblemName), val);			\
	}

#endif	/* BOMB_REPORT_LINE */
/* ========================================================================== 
 *
 *	Structs, types, etc. for c generic bombs and shields
 *
 * ========================================================================== */

typedef	CShield *	CShieldStar;
typedef	void *		voidStar;

typedef union CValue {
	int		cValue_int;
	CShieldStar	cValue_CShieldStar;
	voidStar	cValue_voidStar;
} CValue;

typedef	void (* ActionFuncP)(
#ifdef	IN_CBOMBX_CXX
	  SourceOfDetonationSignal	aSource
	, CValue			aCharge
#endif	/* IN_CBOMBX_CXX */
);

/* ========================================================================== 
 *
 *	C++ functions to support C bombs
 *
 * ========================================================================== */

#ifdef	IN_CBOMBX_CXX
#define	DECLARE_CBOMB_ARM_FUNCTION(TYPE)	\
extern void CAT(cBombArmType_,TYPE)(		\
	  CBomb *	aCBombP			\
	, TYPE		aCharge			\
	)
#else	/* IN_CBOMBX_CXX */
#define	DECLARE_CBOMB_ARM_FUNCTION(TYPE)	\
extern void CAT(cBombArmType_,TYPE)(		\
	)
#endif	/* IN_CBOMBX_CXX */

DECLARE_CBOMB_ARM_FUNCTION(CShieldStar);
DECLARE_CBOMB_ARM_FUNCTION(voidStar);
DECLARE_CBOMB_ARM_FUNCTION(int);

extern void cBombDetonate(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

extern void cBombDisarm(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

extern void cBombCtor(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
		, ActionFuncP	anActionFuncP
#endif	/* IN_CBOMBX_CXX */
	);

extern void cBombDtor(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

extern void cBombAbandonThrough(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aFreeBombP
		, CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

/* ========================================================================== 
 *
 *	C bomb support macros
 *
 * ========================================================================== */

#define	C_DESIGN_BOMB(KIND,TYPE, ACTION)				\
	C_EXPORT_BOMB(KIND,TYPE, ACTION)

#define	C_DESIGN_BOMB_BEGIN(KIND,TYPE)					\
	C_EXPORT_BOMB_BEGIN(KIND,TYPE)

#define	C_DESIGN_BOMB_END						\
	C_EXPORT_BOMB_END

#define	C_EXPORT_BOMB(KIND,TYPE, ACTION)				\
	C_ORDER_BOMB(KIND,TYPE);					\
	C_BUILD_BOMB(KIND,TYPE, ACTION)

#define	C_EXPORT_BOMB_BEGIN(KIND,TYPE)					\
	C_ORDER_BOMB(KIND,TYPE);					\
	C_BUILD_BOMB_BEGIN(KIND,TYPE)

#define	C_EXPORT_BOMB_END						\
	C_BUILD_BOMB_END

#ifdef	IN_CBOMBX_CXX
#define	C_ORDER_BOMB(KIND,TYPE)						\
	typedef	TYPE	CAT(KIND,_type);				\
	void CAT(KIND,_cBombAction)(					\
		  SourceOfDetonationSignal				\
		, CValue						\
	)
#else	/* IN_CBOMBX_CXX */
#define	C_ORDER_BOMB(KIND,TYPE)						\
	typedef	TYPE	CAT(KIND,_type);				\
	void CAT(KIND,_cBombAction)()
#endif	/* IN_CBOMBX_CXX */

#define	C_BUILD_BOMB(KIND,TYPE, ACTION)					\
	C_BUILD_BOMB_BEGIN(KIND,TYPE)					\
		ACTION;							\
	C_BUILD_BOMB_END

#define	C_BUILD_BOMB_BEGIN(KIND,TYPE)					\
	void CAT(KIND,_cBombAction)(SOURCE, aValueStruct)		\
		SourceOfDetonationSignal	SOURCE;			\
		CValue				aValueStruct;		\
	{								\
		TYPE	CHARGE = aValueStruct.CAT(cValue_,TYPE);

#define	C_BUILD_BOMB_END						\
	}


#define	C_DESIGN_SMART_BOMB(KIND,TYPE, ACTION)				\
	C_EXPORT_SMART_BOMB(KIND,TYPE, ACTION)

#define	C_DESIGN_SMART_BOMB_BEGIN(KIND,TYPE)				\
	C_EXPORT_SMART_BOMB_BEGIN(KIND,TYPE)

#define	C_DESIGN_SMART_BOMB_END						\
	C_EXPORT_SMART_BOMB_END

#define	C_EXPORT_SMART_BOMB(KIND,TYPE, ACTION)				\
	C_EXPORT_BOMB(KIND,TYPE, ACTION)

#define	C_EXPORT_SMART_BOMB_BEGIN(KIND,TYPE)				\
	C_EXPORT_BOMB_BEGIN(KIND,TYPE)

#define	C_EXPORT_SMART_BOMB_END						\
	C_EXPORT_BOMB_END

#define	C_PLANT_BOMB(KIND, TAG, BLOCK)					\
	C_PLANT_BOMB_BEGIN(KIND, TAG)					\
		BLOCK							\
	C_PLANT_BOMB_END

#define	C_PLANT_BOMB_BEGIN(KIND, TAG)					\
	{								\
		CBomb		CAT(TAG,_cBombTag);			\
		CBomb *		_innermost_bomb = & CAT(TAG,_cBombTag);	\
		cBombCtor(&CAT(TAG,_cBombTag), CAT(KIND,_cBombAction));

#define	C_PLANT_BOMB_END						\
		cBombDtor(_innermost_bomb);				\
	}

#define	C_ARM_BOMB(TAG,TYPE,value)					\
	CAT(cBombArmType_,TYPE)(&CAT(TAG,_cBombTag), value)

#define	C_DETONATE_BOMB(TAG)						\
	cBombDetonate(&CAT(TAG,_cBombTag))

#define	C_DISARM_BOMB(TAG)						\
	cBombDisarm(&CAT(TAG,_cBombTag))

/* ========================================================================== 
 *
 *	C shield support functions
 *
 * ========================================================================== */

void cShieldBombCtor(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

void cLShieldBombCtor(
#ifdef	IN_CBOMBX_CXX
		  CBomb *	aCBombP
#endif	/* IN_CBOMBX_CXX */
	);

void cSetUpErrorListAndSavedPtr(
#ifdef	IN_CBOMBX_CXX
	  CShield *
	, char **
#endif	/* IN_CBOMBX_CXX */
);

jmp_buf * cFetchJmpBuf(
#ifdef	IN_CBOMBX_CXX
	  CShield *
#endif	/* IN_CBOMBX_CXX */
);

void * cFetchProblemP(
#ifdef	IN_CBOMBX_CXX
	  CShield *
#endif	/* IN_CBOMBX_CXX */
);

#ifdef	IN_CBOMBX_CXX
CONST
#endif	/* IN_CBOMBX_CXX */
char * cProblemFetchProblemName(
#ifdef	IN_CBOMBX_CXX
	  void *
#endif	/* IN_CBOMBX_CXX */
);

int cProblemFetchVal(
#ifdef	IN_CBOMBX_CXX
	  void *
#endif	/* IN_CBOMBX_CXX */
);

#ifdef	IN_CBOMBX_CXX
CONST
#endif	/* IN_CBOMBX_CXX */
char * cProblemFetchFileName(
#ifdef	IN_CBOMBX_CXX
	  void *
#endif	/* IN_CBOMBX_CXX */
);

int cProblemFetchLineNumber(
#ifdef	IN_CBOMBX_CXX
	  void *
#endif	/* IN_CBOMBX_CXX */
);

/* ========================================================================== 
 *
 *	C shield
 *
 * ========================================================================== */

/*  C shields don't use C_ORDER_BOMB directly, for a different reason
 *  than c++ shields.  They don't need to be tweaked to override the
 *  inspectFuse() member function and don't need to define the action
 *  routine (since both are provided by the fake constructor).
 *
 *  Unlike c++ shields they can't just use C_PLANT_BOMB()
 *  because they must call a different constructor.
 */

#define	C_INSTALL_SHIELD(TAG, BLOCK)					\
	C_INSTALL_SHIELD_BEGIN(TAG)					\
		BLOCK							\
	C_INSTALL_SHIELD_END

#define	C_INSTALL_SHIELD_BEGIN(TAG)					\
	{								\
		CShield		CAT(TAG,_cShield);			\
		CBomb		CAT(TAG,_cBombTag);			\
		CBomb *		_innermost_bomb = & CAT(TAG,_cBombTag);	\
									\
		cShieldCtor(&CAT(TAG,_cShield));			\
		cShieldBombCtor(&CAT(TAG,_cBombTag));

#define	C_INSTALL_SHIELD_END						\
	C_PLANT_BOMB_END

#define	C_INSTALL_LOUD_SHIELD(TAG, BLOCK)				\
	C_INSTALL_LOUD_SHIELD_BEGIN(TAG)				\
		BLOCK							\
	C_INSTALL_LOUD_SHIELD_END

#define	C_INSTALL_LOUD_SHIELD_BEGIN(TAG)				\
	{								\
		CShield		CAT(TAG,_cShield);			\
		CBomb		CAT(TAG,_cBombTag);			\
		CBomb *		_innermost_bomb = & CAT(TAG,_cBombTag);	\
									\
		cShieldCtor(&CAT(TAG,_cShield));			\
		cLShieldBombCtor(&CAT(TAG,_cBombTag));

#define	C_INSTALL_LOUD_SHIELD_END					\
	C_PLANT_BOMB_END

/* ==========================================================================
 *
 *	Now that C_ORDER_BOMB() is defined, we can expand it for SHIELDS
 *
 *	(Type is actually Problem *)
 *
 * ========================================================================== */

	C_ORDER_BOMB(CShieldFree, voidStar);

/* ==========================================================================
 *
 *	C_SHIELD_UP()	Raise (activate) the shield.
 *
 *	(The ShieldFree bomb is needed to free the storage allocated by
 *	 incoming letter bombs.)
 *
 * ========================================================================== */

#define	C_SHIELD_UP(TAG, LTAG, ACTION)					\
	C_SHIELD_UP_BEGIN(TAG, LTAG) {					\
	ACTION								\
	} C_SHIELD_UP_END(TAG)

#define	C_SHIELD_UP_BEGIN(TAG, LTAG)					\
	{								\
		int	VAL;						\
									\
		cSetUpErrorListAndSavedPtr(&CAT(TAG,_cShield),		\
			CAT(LTAG,_shieldList)());				\
									\
		if ((VAL = setjmp(*cFetchJmpBuf(&CAT(TAG,_cShield))))) { \
			void *	C_PROBLEM_P = cFetchProblemP(&CAT(TAG,_cShield)); \
			C_PLANT_BOMB_BEGIN(CShieldFree,_cSheildFree) { 	\
				C_ARM_BOMB(_cSheildFree,voidStar,C_PROBLEM_P); \
				{

#define	C_SHIELD_UP_END(TAG)						\
				}					\
			} C_PLANT_BOMB_END				\
		}							\
	}								\
	C_ARM_BOMB(TAG,CShieldStar,&CAT(TAG,_cShield))

/* ==========================================================================
 *
 *	C_SHIELD_DOWN()	Lower (deactivate) the shield.
 *
 * ========================================================================== */

#define	C_SHIELD_DOWN(TAG)						\
		C_DISARM_BOMB(TAG)

/* ==========================================================================
 *
 *	C_REBLAST()	Problem object in current shield.
 *
 * ========================================================================== */

#define	C_REBLAST	cReblast(C_PROBLEM_P);

/* ==========================================================================
 *
 *		C_RETURN_VOID
 *
 *		C_RETURN_THROUGH(TAG,RETVAL)
 *		C_BREAK_THROUGH(TAG)
 *		C_CONTINUE_THROUGH(TAG)
 *
 *		C_BREAK_FROM_SHIELD
 *		C_CONTINUE_FROM_SHIELD
 *
 *	In the c++ version of the demolition package, an unconditional return
 *	inside a shield causes the ShieldFree bomb's destructor to be
 *	unreachable, so the compiler will warn us of this.  The analog of
 *	"C_SHIELD_RETURN...(retval);" may be used in place of
 *	"return retval;"  to shut the compiler up.  (The if() always takes
 *	the TRUE branch, but convinces the compiler that the other branch
 *	is possible.  A good enough optimizer will squeeze it out, but still
 *	not complain.)
 *
 *	C_RETURN_VOID is something to be returned in C_RETURN_THROUGH() when
 *	used in a routine that doesn't return anything.
 *
 *	Similarly with "break;" and "C_BREAK...;"
 *	Similarly with "continue;" and "C_CONTINUE...;"
 *
 *	In the c version, the problem is worse:
 *
 *	 - C does not automatically generate destructors for us, so we
 *	   must do something to activate them ourselves.  This means we
 *	   MUST use this stuff:
 *	    - not just to keep the warning messages down, but to generate
 *	      correct code.
 *	    - not just when we're leaving a shield, but whenever flow
 *	      passes through shields or bombs via a "return", "break" or
 *	      "continue".
 *
 *       - We must generate destructor activity, not just for the innermost
 *	   bomb or shield, but or all of those we pass through on our way out.
 *	   The support package keeps track of the nesting for us, but we must
 *	   designate the outermost shield or bomb that flow will leave.
 *
 *	So we have the more extensive set of routines:
 *
 *	 - C_RETURN_THROUGH(TAG,RETVAL)	\  Used to leave a shield's ACTION,
 *	 - C_BREAK_THROUGH(TAG)		 } where TAG is from the outermost
 *	 - C_CONTINUE_THROUGH(TAG)	/  INSTALL/PLANT flow is leaving.
 *
 *	 - C_BREAK_FROM_SHIELD		\  Used to leave a shield's ACTION
 *	 - C_CONTINUE_FROM_SHIELD	/  without leaving any INSTALL/PLANTs
 *
 *	NOTE that the compiler will not give warnings for most failures to
 *	use these macros.
 *
 *	NOTE that if TAGs are reused in a nest, these macros will designate
 *	the innermost one, and there is no way to detect the potential error.
 *
 * ========================================================================== */

#define C_RETURN_VOID

#define C_WHATEVER_THROUGH(TAG,WHATEVER)				\
	cBombAbandonThrough(_innermost_bomb, &CAT(TAG,_cBombTag));	\
	if (&CAT(TAG,_cBombTag)) { WHATEVER; }

#define C_RETURN_THROUGH(TAG,RETVAL)					\
		C_WHATEVER_THROUGH(TAG,return RETVAL)

#define C_BREAK_THROUGH(TAG)						\
		C_WHATEVER_THROUGH(TAG,break)

#define C_CONTINUE_THROUGH(TAG)						\
		C_WHATEVER_THROUGH(TAG,continue)

#define C_BREAK_FROM_SHIELD						\
		C_BREAK_THROUGH(_cSheildFree)

#define C_CONTINUE_FROM_SHIELD						\
		C_CONTINUE_THROUGH(_cSheildFree)

/* ==========================================================================
 *
 *	A construct useful mainly for test files:
 *
 * ==========================================================================
 *
 *	C_BOOBY_TRAP()	Do {ACTION}.  It should BLAST(ONE_OF_LTAG).
 *			 - If it does, catch it and go on.
 *			 - If it doesn't, BLAST(GRIPE).
 *			{ Requires a C_INSTALL_SHIELD(C_BOOBY_TRAPPING) }
 *
 * ========================================================================== */

#define	C_BOOBY_TRAP(LTAG,GRIPE,ACTION)				\
	C_BOOBY_TRAP_BEGIN(LTAG)				\
		ACTION						\
	C_BOOBY_TRAP_END(GRIPE)

#define	C_BOOBY_TRAP_BEGIN(LTAG)				\
	for (;;) {						\
		C_SHIELD_UP(C_BOOBY_TRAPPING,LTAG,{		\
	/**/		C_BREAK_FROM_SHIELD			\
		});

#define	C_BOOBY_TRAP_END(GRIPE)					\
		C_BLAST(GRIPE);					\
	}

/* ========================================================================== 
 *
 *	DOOMSDAY BOMBs are a special case of SHIELD that brings down the
 *	application if the SHIELD would have caught a BLAST().
 *
 * ==========================================================================
 *
 *	PLANT_DOOMSDAY_BOMB()	Instantiate a doomsday bomb object.
 *
 * ========================================================================== */

#define	C_PLANT_DOOMSDAY_BOMB(BLOCK)					\
	C_PLANT_DOOMSDAY_BOMB_BEGIN					\
		BLOCK							\
	C_PLANT_DOOMSDAY_BOMB_END

#define	C_PLANT_DOOMSDAY_BOMB_BEGIN					\
	C_INSTALL_SHIELD_BEGIN(_DOOMSDAY)

#define	C_PLANT_DOOMSDAY_BOMB_END					\
	C_INSTALL_SHIELD_END

/* ==========================================================================
 *
 *	C_ARM_DOOMSDAY_BOMB()
 *
 *	Logically identical to: C_SHIELD_UP(_DOOMSDAY, _DOOMSDAY, {;})
 *	but it never really gets jumped to, so we simplify it massively
 *	(eliminating the setjmp() and the guts) to save time and space.
 *
 * ========================================================================== */

#define	C_ARM_DOOMSDAY_BOMB()						\
	{								\
		cSetUpErrorListAndSavedPtr(&CAT(_DOOMSDAY,_cShield),	\
			CAT(_DOOMSDAY,_shieldList)());			\
	}								\
	C_ARM_BOMB(_DOOMSDAY,CShieldStar,&CAT(_DOOMSDAY,_cShield))

/* ==========================================================================
 *
 *	DISARM_DOOMSDAY_BOMB()
 *
 * ========================================================================== */

#define	C_DISARM_DOOMSDAY_BOMB()	C_SHIELD_DOWN(_DOOMSDAY)

extern char **	CAT(_DOOMSDAY,_shieldList)();

#endif /* CBOMBC_H */
