/* ========================================================================== */
//
//	Copyright (c) 1989 by Xanadu Operating Company.  All rights reserved.
//	Copyright (c) Xerox Corporation 1989.  All rights reserved.
//
//	NOT TO BE EXPORTED IN ANY FORM FROM THE UNITED STATES
//	WITHOUT ADVANCE APPROVAL OF THE U.S. GOVERNMENT.
//
/* ========================================================================== */
//
// The information contained herein is confidential, proprietary to Xanadu
// Operating Company, and considered a trade secret as defined in section
// 499C of the penal code of the State of California.
//
// Use of this information by anyone other than authorized employees of
// Xanadu is granted only under a written nondisclosure agreement,
// expressly prescribing the scope and manner of such use.
//
// The above copyright notice is not to be construed as evidence of
// publication or the intent to publish.
//
// The code in this module is derived from the Xerox Secure Hash Function.
// It is used under license from the Xerox Corporation.  (See below.)
// (Contact Xerox Corporation for a version uncontaminated by Xanadu
//  proprietary code.)
//
/* ========================================================================== */
//
//				SBox.cxx
//
//	X++ S-boxes for Merkle's "Pharonic cypher" Cryptographic hash.
//
/* ========================================================================== */

/*        This is the standard S box used by the one-way hash function Snefru.
           See the paper 'A Software Encryption Function', by Ralph C.  Merkle,
          for a more detailed explanation.

Copyright (c) Xerox Corporation 1989
All rights reserved.

License to  copy and use this software is granted provided that it is identified
as the 'Tables For the Xerox Secure Hash Function' in all material mentioning
or referencing this software.

License is also granted to make and use derivative works provided that such
works are identified as 'derived from the Tables For the Xerox Secure Hash
Function' in all material mentioning or referencing the derived work.

Xerox Corporation makes no representations concerning either the
merchantability of this software or the suitability of this software for any
particular purpose.  It is provided "as is" without express or implied
warranty of any kind.

Released for distribution in the United States only.
NOT TO BE EXPORTED IN ANY FORM FROM THE UNITED STATES
WITHOUT ADVANCE APPROVAL OF THE U.S. GOVERNMENT.

These notices must be retained in any copies of any part of this software.



Implementor:  Ralph C. Merkle

Ported to X++ by Mark S. Miller and Eric C. Hill
Hacked for readability by Michael J. McClary

*/

static char * sboxx_cxx_version() {
    return "$Id: sboxx.cxx,v 2.2 1992/06/05 23:43:31 eric Exp $";
}

#include "sboxx.hxx"


SBox standardSBoxes[maxSBoxCount] = {

{  /*  Start of S Box 1, also known as the initial S-box  */
  0x64F9001B,  /*  0  */
  0xFEDDCDF6,  /*  1  */
  0x7C8FF1E2,  /*  2  */
  0x11D71514,  /*  3  */
  0x8B8C18D3,  /*  4  */
  0xDDDF881E,  /*  5  */
  0x6EAB5056,  /*  6  */
  0x88CED8E1,  /*  7  */
  0x49148959,  /*  8  */
  0x69C56FD5,  /*  9  */
  0xB7994F03,  /*  10  */
  0xFBCEE3E,  /*  11  */
  0x3C264940,  /*  12  */
  0x21557E58,  /*  13  */
  0xE14B3FC2,  /*  14  */
  0x2E5CF591,  /*  15  */
  0xDCEFF8CE,  /*  16  */
  0x92A1648,  /*  17  */
  0xBE812936,  /*  18  */
  0xFF7B0C6A,  /*  19  */
  0xD5251037,  /*  20  */
  0xAFA448F1,  /*  21  */
  0x7DAFC95A,  /*  22  */
  0x1EA69C3F,  /*  23  */
  0xA417ABE7,  /*  24  */
  0x5890E423,  /*  25  */
  0xB0CB70C0,  /*  26  */
  0xC85025F7,  /*  27  */
  0x244D97E3,  /*  28  */
  0x1FF3595F,  /*  29  */
  0xC4EC6396,  /*  30  */
  0x59181E17,  /*  31  */
  0xE635B477,  /*  32  */
  0x354E7DBF,  /*  33  */
  0x796F7753,  /*  34  */
  0x66EB52CC,  /*  35  */
  0x77C3F995,  /*  36  */
  0x32E3A927,  /*  37  */
  0x80CCAED6,  /*  38  */
  0x4E2BE89D,  /*  39  */
  0x375BBD28,  /*  40  */
  0xAD1A3D05,  /*  41  */
  0x2B1B42B3,  /*  42  */
  0x16C44C71,  /*  43  */
  0x4D54BFA8,  /*  44  */
  0xE57DDC7A,  /*  45  */
  0xEC6D8144,  /*  46  */
  0x5A71046B,  /*  47  */
  0xD8229650,  /*  48  */
  0x87FC8F24,  /*  49  */
  0xCBC60E09,  /*  50  */
  0xB6390366,  /*  51  */
  0xD9F76092,  /*  52  */
  0xD393A70B,  /*  53  */
  0x1D31A08A,  /*  54  */
  0x9CD971C9,  /*  55  */
  0x5C1EF445,  /*  56  */
  0x86FAB694,  /*  57  */
  0xFDB44165,  /*  58  */
  0x8EAAFCBE,  /*  59  */
  0x4BCAC6EB,  /*  60  */
  0xFB7A94E5,  /*  61  */
  0x5789D04E,  /*  62  */
  0xFA13CF35,  /*  63  */
  0x236B8DA9,  /*  64  */
  0x4133F000,  /*  65  */
  0x6224261C,  /*  66  */
  0xF412F23B,  /*  67  */
  0xE75E56A4,  /*  68  */
  0x30022116,  /*  69  */
  0xBAF17F1F,  /*  70  */
  0xD09872F9,  /*  71  */
  0xC1A3699C,  /*  72  */
  0xF1E802AA,  /*  73  */
  0xDD145DC,  /*  74  */
  0x4FDCE093,  /*  75  */
  0x8D8412F0,  /*  76  */
  0x6CD0F376,  /*  77  */
  0x3DE6B73D,  /*  78  */
  0x84BA737F,  /*  79  */
  0xB43A30F2,  /*  80  */
  0x44569F69,  /*  81  */
  0xE4EACA,  /*  82  */
  0xB58DE3B0,  /*  83  */
  0x959113C8,  /*  84  */
  0xD62EFEE9,  /*  85  */
  0x90861F83,  /*  86  */
  0xCED69874,  /*  87  */
  0x2F793CEE,  /*  88  */
  0xE8571C30,  /*  89  */
  0x483665D1,  /*  90  */
  0xAB07B031,  /*  91  */
  0x914C844F,  /*  92  */
  0x15BF3BE8,  /*  93  */
  0x2C3F2A9A,  /*  94  */
  0x9EB95FD4,  /*  95  */
  0x92E7472D,  /*  96  */
  0x2297CC5B,  /*  97  */
  0xEE5F2782,  /*  98  */
  0x5377B562,  /*  99  */
  0xDB8EBBCF,  /*  100  */
  0xF961DEDD,  /*  101  */
  0xC59B5C60,  /*  102  */
  0x1BD3910D,  /*  103  */
  0x26D206AD,  /*  104  */
  0xB28514D8,  /*  105  */
  0x5ECF6B52,  /*  106  */
  0x7FEA78BB,  /*  107  */
  0x504879AC,  /*  108  */
  0xED34A884,  /*  109  */
  0x36E51D3C,  /*  110  */
  0x1753741D,  /*  111  */
  0x8C47CAED,  /*  112  */
  0x9D0A40EF,  /*  113  */
  0x3145E221,  /*  114  */
  0xDA27EB70,  /*  115  */
  0xDF730BA3,  /*  116  */
  0x183C8789,  /*  117  */
  0x739AC0A6,  /*  118  */
  0x9A58DFC6,  /*  119  */
  0x54B134C1,  /*  120  */
  0xAC3E242E,  /*  121  */
  0xCC493902,  /*  122  */
  0x7B2DDA99,  /*  123  */
  0x8F15BC01,  /*  124  */
  0x29FD38C7,  /*  125  */
  0x27D5318F,  /*  126  */
  0x604AAFF5,  /*  127  */
  0xF29C6818,  /*  128  */
  0xC38AA2EC,  /*  129  */
  0x1019D4C3,  /*  130  */
  0xA8FB936E,  /*  131  */
  0x20ED7B39,  /*  132  */
  0xB686119,  /*  133  */
  0x89A0906F,  /*  134  */
  0x1CC7829E,  /*  135  */
  0x9952EF4B,  /*  136  */
  0x850E9E8C,  /*  137  */
  0xCD063A90,  /*  138  */
  0x67002F8E,  /*  139  */
  0xCFAC8CB7,  /*  140  */
  0xEAA24B11,  /*  141  */
  0x988B4E6C,  /*  142  */
  0x46F066DF,  /*  143  */
  0xCA7EEC08,  /*  144  */
  0xC7BBA664,  /*  145  */
  0x831D17BD,  /*  146  */
  0x63F575E6,  /*  147  */
  0x9764350E,  /*  148  */
  0x47870D42,  /*  149  */
  0x26CA4A2,  /*  150  */
  0x8167D587,  /*  151  */
  0x61B6ADAB,  /*  152  */
  0xAA6564D2,  /*  153  */
  0x70DA237B,  /*  154  */
  0x25E1C74A,  /*  155  */
  0xA1C901A0,  /*  156  */
  0xEB0A5DA,  /*  157  */
  0x7670F741,  /*  158  */
  0x51C05AEA,  /*  159  */
  0x933DFA32,  /*  160  */
  0x759FF1A,  /*  161  */
  0x56010AB8,  /*  162  */
  0x5FDECB78,  /*  163  */
  0x3F32EDF8,  /*  164  */
  0xAEBEDBB9,  /*  165  */
  0x39F8326D,  /*  166  */
  0xD20858C5,  /*  167  */
  0x9B638BE4,  /*  168  */
  0xA572C80A,  /*  169  */
  0x28E0A19F,  /*  170  */
  0x432099FC,  /*  171  */
  0x3A37C3CD,  /*  172  */
  0xBF95C585,  /*  173  */
  0xB392C12A,  /*  174  */
  0x6AA707D7,  /*  175  */
  0x52F66A61,  /*  176  */
  0x12D483B1,  /*  177  */
  0x96435B5E,  /*  178  */
  0x3E75802B,  /*  179  */
  0x3BA52B33,  /*  180  */
  0xA99F51A5,  /*  181  */
  0xBDA1E157,  /*  182  */
  0x78C2E70C,  /*  183  */
  0xFCAE7CE0,  /*  184  */
  0xD1602267,  /*  185  */
  0x2AFFAC4D,  /*  186  */
  0x4A510947,  /*  187  */
  0xAB2B83A,  /*  188  */
  0x7A04E579,  /*  189  */
  0x340DFD80,  /*  190  */
  0xB916E922,  /*  191  */
  0xE29D5E9B,  /*  192  */
  0xF5624AF4,  /*  193  */
  0x4CA9D9AF,  /*  194  */
  0x6BBD2CFE,  /*  195  */
  0xE3B7F620,  /*  196  */
  0xC2746E07,  /*  197  */
  0x5B42B9B6,  /*  198  */
  0xA06919BC,  /*  199  */
  0xF0F2C40F,  /*  200  */
  0x72217AB5,  /*  201  */
  0x14C19DF3,  /*  202  */
  0xF3802DAE,  /*  203  */
  0xE094BEB4,  /*  204  */
  0xA2101AFF,  /*  205  */
  0x529575D,  /*  206  */
  0x55CDB27C,  /*  207  */
  0xA33BDDB2,  /*  208  */
  0x6528B37D,  /*  209  */
  0x740C05DB,  /*  210  */
  0xE96A62C4,  /*  211  */
  0x40782846,  /*  212  */
  0x6D30D706,  /*  213  */
  0xBBF48E2C,  /*  214  */
  0xBCE2D3DE,  /*  215  */
  0x49E37FA,  /*  216  */
  0x1B5E634,  /*  217  */
  0x2D886D8D,  /*  218  */
  0x7E5A2E7E,  /*  219  */
  0xD7412013,  /*  220  */
  0x6E90F97,  /*  221  */
  0xE45D3EBA,  /*  222  */
  0xB8AD3386,  /*  223  */
  0x13051B25,  /*  224  */
  0xC035354,  /*  225  */
  0x71C89B75,  /*  226  */
  0xC638FBD0,  /*  227  */
  0x197F11A1,  /*  228  */
  0xEF0F08FB,  /*  229  */
  0xF8448651,  /*  230  */
  0x38409563,  /*  231  */
  0x452F4443,  /*  232  */
  0x5D464D55,  /*  233  */
  0x3D8764C,  /*  234  */
  0xB1B8D638,  /*  235  */
  0xA70BBA2F,  /*  236  */
  0x94B3D210,  /*  237  */
  0xEB6692A7,  /*  238  */
  0xD409C2D9,  /*  239  */
  0x68838526,  /*  240  */
  0xA6DB8A15,  /*  241  */
  0x751F6C98,  /*  242  */
  0xDE769A88,  /*  243  */
  0xC9EE4668,  /*  244  */
  0x1A82A373,  /*  245  */
  0x896AA49,  /*  246  */
  0x42233681,  /*  247  */
  0xF62C55CB,  /*  248  */
  0x9F1C5404,  /*  249  */
  0xF74FB15C,  /*  250  */
  0xC06E4312,  /*  251  */
  0x6FFE5D72,  /*  252  */
  0x8AA8678B,  /*  253  */
  0x337CD129,  /*  254  */
  0x8211CEFD  /*  255  */
  /*  End of S Box 1  */ },


{  /*  Start of S Box 2  */
  0x61B0B02F,  /*  0  */
  0xE27716,  /*  1  */
  0xBF32D884,  /*  2  */
  0x6FA356FF,  /*  3  */
  0x35842720,  /*  4  */
  0x54607261,  /*  5  */
  0x7828C5AE,  /*  6  */
  0x294211CF,  /*  7  */
  0x4E81528B,  /*  8  */
  0xDD5457A7,  /*  9  */
  0xD9D32BE,  /*  10  */
  0xAF55B23F,  /*  11  */
  0x3F8699A0,  /*  12  */
  0xDBB4AF42,  /*  13  */
  0xD744DC65,  /*  14  */
  0xC93ECF0,  /*  15  */
  0xE359680B,  /*  16  */
  0x46DF2EF,  /*  17  */
  0x1BF24487,  /*  18  */
  0x595146D2,  /*  19  */
  0x8BDFD42B,  /*  20  */
  0x2B0D38EA,  /*  21  */
  0xAC18A09F,  /*  22  */
  0x73AF8D78,  /*  23  */
  0x68AEBA06,  /*  24  */
  0xC5FFA500,  /*  25  */
  0x8DB36B2D,  /*  26  */
  0xA040D9B7,  /*  27  */
  0x8583C012,  /*  28  */
  0xB5FC22E3,  /*  29  */
  0x239714D5,  /*  30  */
  0xA795C69A,  /*  31  */
  0xB43FBCDB,  /*  32  */
  0x628D4AE0,  /*  33  */
  0x187A5941,  /*  34  */
  0xA9491C34,  /*  35  */
  0x77CB9482,  /*  36  */
  0x14530C67,  /*  37  */
  0x478A1253,  /*  38  */
  0x754AE323,  /*  39  */
  0x7EA8C1E7,  /*  40  */
  0x9087BFAF,  /*  41  */
  0xD219BBE5,  /*  42  */
  0x608500CB,  /*  43  */
  0x3E6940A4,  /*  44  */
  0x3B7D5F5B,  /*  45  */
  0xB95E86F7,  /*  46  */
  0x8CDC2E1,  /*  47  */
  0x2F7AD77,  /*  48  */
  0x40B5DAF1,  /*  49  */
  0x8E725492,  /*  50  */
  0x740FA82C,  /*  51  */
  0x3094F8E8,  /*  52  */
  0xC925810A,  /*  53  */
  0x89FA96C6,  /*  54  */
  0x7C4375D1,  /*  55  */
  0x2F36B383,  /*  56  */
  0xA461C74F,  /*  57  */
  0x4426906D,  /*  58  */
  0x20093EDA,  /*  59  */
  0x2C41DFD8,  /*  60  */
  0xD8E06097,  /*  61  */
  0xD08B1A03,  /*  62  */
  0x5EBBEA72,  /*  63  */
  0xAD560825,  /*  64  */
  0x795B1388,  /*  65  */
  0x7D6F4DFE,  /*  66  */
  0xE7D9F74E,  /*  67  */
  0xEC8919A1,  /*  68  */
  0x82F0B55F,  /*  69  */
  0xB70FCD4,  /*  70  */
  0xC433D7C7,  /*  71  */
  0x305C3B8,  /*  72  */
  0x9CCC826E,  /*  73  */
  0x9B0A5848,  /*  74  */
  0xB3A47130,  /*  75  */
  0x633D24ED,  /*  76  */
  0x3DDCE217,  /*  77  */
  0xB8376E76,  /*  78  */
  0x139FDAC,  /*  79  */
  0x5CA23769,  /*  80  */
  0x6BBE5B59,  /*  81  */
  0x883BAEBB,  /*  82  */
  0x21D6E605,  /*  83  */
  0x53B623CD,  /*  84  */
  0xF5F1CB80,  /*  85  */
  0x4ACF04C2,  /*  86  */
  0x4C3A91B4,  /*  87  */
  0xDA45E998,  /*  88  */
  0xB15F7B24,  /*  89  */
  0x57A74CBC,  /*  90  */
  0x674E0B1D,  /*  91  */
  0xB0D18A3D,  /*  92  */
  0xE9D0677B,  /*  93  */
  0x382F9DC0,  /*  94  */
  0x34031701,  /*  95  */
  0x1A9B1B02,  /*  96  */
  0xE5DE7DA9,  /*  97  */
  0x33C4A173,  /*  98  */
  0x4D1B8B0F,  /*  99  */
  0x376E2F6F,  /*  100  */
  0x247E2EEE,  /*  101  */
  0x27F6CFA8,  /*  102  */
  0x2814E04D,  /*  103  */
  0x155AE77E,  /*  104  */
  0x2592483C,  /*  105  */
  0x17473143,  /*  106  */
  0x9923CD46,  /*  107  */
  0xDF1A06D9,  /*  108  */
  0x724F69E6,  /*  109  */
  0xB76B70FA,  /*  110  */
  0xB6A536FB,  /*  111  */
  0xF4EE0951,  /*  112  */
  0xEE357EEC,  /*  113  */
  0x874DB952,  /*  114  */
  0x52579C1F,  /*  115  */
  0xFEE40F32,  /*  116  */
  0x5D4B3009,  /*  117  */
  0xBC71EF8A,  /*  118  */
  0xFDAC884B,  /*  119  */
  0x4B9C50CC,  /*  120  */
  0xF72BB88E,  /*  121  */
  0x7B0B974C,  /*  122  */
  0xABD3CC8C,  /*  123  */
  0x58C961C5,  /*  124  */
  0x2D734228,  /*  125  */
  0x5AFB05BF,  /*  126  */
  0xAA766227,  /*  127  */
  0x76E8BDF9,  /*  128  */
  0xC70E291E,  /*  129  */
  0xFB48189D,  /*  130  */
  0xCAF40A79,  /*  131  */
  0xF298C9E,  /*  132  */
  0x9F1CF3D3,  /*  133  */
  0x658E7470,  /*  134  */
  0xC2CE3B26,  /*  135  */
  0x6D04D3DF,  /*  136  */
  0x8CC7808D,  /*  137  */
  0x92BD43C9,  /*  138  */
  0xA88CB490,  /*  139  */
  0x5B52840D,  /*  140  */
  0x32D8FF58,  /*  141  */
  0xD9B27FC8,  /*  142  */
  0xED22C15,  /*  143  */
  0x7A636F5A,  /*  144  */
  0x56999E18,  /*  145  */
  0xF050DBF2,  /*  146  */
  0x42675129,  /*  147  */
  0xA23C2A1A,  /*  148  */
  0x98669B19,  /*  149  */
  0x1E881540,  /*  150  */
  0xD3F589F3,  /*  151  */
  0xD164F55C,  /*  152  */
  0x13CAD545,  /*  153  */
  0xF3AAF0AB,  /*  154  */
  0x5FA1D0B6,  /*  155  */
  0x9AEC4908,  /*  156  */
  0xA5E53AA2,  /*  157  */
  0xE212A254,  /*  158  */
  0x9D6CF60E,  /*  159  */
  0x2E901E6B,  /*  160  */
  0x12796C36,  /*  161  */
  0xC8C283AD,  /*  162  */
  0x411D2163,  /*  163  */
  0x7D7E435,  /*  164  */
  0x4611DE74,  /*  165  */
  0xA6965A93,  /*  166  */
  0xE4342660,  /*  167  */
  0x26C3E57F,  /*  168  */
  0xF11095F5,  /*  169  */
  0x93A98F91,  /*  170  */
  0xBA2EB74A,  /*  171  */
  0xC365EE04,  /*  172  */
  0x5538A7E9,  /*  173  */
  0x1D75DD3E,  /*  174  */
  0xCC3153E4,  /*  175  */
  0xBB77E821,  /*  176  */
  0x91B70E44,  /*  177  */
  0x691D65E,  /*  178  */
  0xDE165CA6,  /*  179  */
  0xA378C968,  /*  180  */
  0xCD1334D0,  /*  181  */
  0xCE006694,  /*  182  */
  0x710C283A,  /*  183  */
  0xE62DC87A,  /*  184  */
  0x1C1EF97D,  /*  185  */
  0xE0AB92AA,  /*  186  */
  0xB246792E,  /*  187  */
  0x4FB9FADE,  /*  188  */
  0x5174AC1C,  /*  189  */
  0x69682557,  /*  190  */
  0xE8985537,  /*  191  */
  0x45A66DA3,  /*  192  */
  0x961F4B99,  /*  193  */
  0xDC7F3CF8,  /*  194  */
  0x4902020C,  /*  195  */
  0x6E17FB7C,  /*  196  */
  0xC0FD8747,  /*  197  */
  0xCB8FEB6A,  /*  198  */
  0x8458CEB9,  /*  199  */
  0x64BCF196,  /*  200  */
  0x1021AAD6,  /*  201  */
  0x50C01FB3,  /*  202  */
  0xAEC5D175,  /*  203  */
  0x864C9A22,  /*  204  */
  0x7FE3851B,  /*  205  */
  0x3CB8AB33,  /*  206  */
  0x319A4EFD,  /*  207  */
  0xEBC66513,  /*  208  */
  0xE1E9E164,  /*  209  */
  0x48B13585,  /*  210  */
  0x8FDDA3E2,  /*  211  */
  0x36D41D07,  /*  212  */
  0x70E19F5D,  /*  213  */
  0xC1273F39,  /*  214  */
  0x836A98B1,  /*  215  */
  0xA1303371,  /*  216  */
  0x3A2C7881,  /*  217  */
  0xA5D1686,  /*  218  */
  0xD5060150,  /*  219  */
  0x5C8B68F,  /*  220  */
  0xBEF8A6C1,  /*  221  */
  0x2A820311,  /*  222  */
  0x80BA4F10,  /*  223  */
  0xEA9F76CA,  /*  224  */
  0xEF80CA3B,  /*  225  */
  0xD47C4755,  /*  226  */
  0xBD07ED89,  /*  227  */
  0xFAC17A95,  /*  228  */
  0x399E2B56,  /*  229  */
  0x16DAD26C,  /*  230  */
  0xCF2A41A5,  /*  231  */
  0x6AADB1EB,  /*  232  */
  0x81DB5EF6,  /*  233  */
  0x6CE62D9B,  /*  234  */
  0x9EEF07FC,  /*  235  */
  0x666210BD,  /*  236  */
  0x94202049,  /*  237  */
  0xF95C3962,  /*  238  */
  0x19A0F4B0,  /*  239  */
  0xF2F963B2,  /*  240  */
  0x97EB7314,  /*  241  */
  0x43086466,  /*  242  */
  0x1FED0DCE,  /*  243  */
  0xF83EFEDC,  /*  244  */
  0xC6E7A431,  /*  245  */
  0x8ABF8E38,  /*  246  */
  0x22F345B5,  /*  247  */
  0x957B5DDD,  /*  248  */
  0xFC1593BA,  /*  249  */
  0x1101A9C3,  /*  250  */
  0xF622BE2A,  /*  251  */
  0xEDFE7CC4,  /*  252  */
  0x9EA3D9C,  /*  253  */
  0xFF24C4D7,  /*  254  */
  0xD6D56AF4  /*  255  */
  /*  End of S Box 2  */ },


{  /*  Start of S Box 3  */
  0x487600CE,  /*  0  */
  0x425C8FFA,  /*  1  */
  0xBEC67996,  /*  2  */
  0xD0CC610C,  /*  3  */
  0xB2A1D01D,  /*  4  */
  0xADAE8C7A,  /*  5  */
  0xAB5D392,  /*  6  */
  0x20B2E45,  /*  7  */
  0x30325588,  /*  8  */
  0x8B630249,  /*  9  */
  0x927320E4,  /*  10  */
  0x2E98A92D,  /*  11  */
  0xB70817DF,  /*  12  */
  0xEB6858AA,  /*  13  */
  0x203186A6,  /*  14  */
  0x225D0823,  /*  15  */
  0x3F66BABE,  /*  16  */
  0x7E4853BF,  /*  17  */
  0x6E96684E,  /*  18  */
  0xE63BD63D,  /*  19  */
  0xD46EC4F8,  /*  20  */
  0x4B2E89A7,  /*  21  */
  0xCA5AD9D,  /*  22  */
  0xF0C39E2E,  /*  23  */
  0x6D866DFB,  /*  24  */
  0x1927FE8A,  /*  25  */
  0xA4DB62D2,  /*  26  */
  0xF4C06E25,  /*  27  */
  0x8A1DB2F9,  /*  28  */
  0xAB7E0F9E,  /*  29  */
  0xFBCB5A79,  /*  30  */
  0x132B6A2A,  /*  31  */
  0x78344DA3,  /*  32  */
  0x8077B50,  /*  33  */
  0xA378DA3F,  /*  34  */
  0xB0E5A681,  /*  35  */
  0x94EF414F,  /*  36  */
  0xAF8894C2,  /*  37  */
  0x52374B69,  /*  38  */
  0x45173E46,  /*  39  */
  0x6AA66044,  /*  40  */
  0x2FBF64FC,  /*  41  */
  0x9185FFC3,  /*  42  */
  0xBC8C07E2,  /*  43  */
  0xDC11196F,  /*  44  */
  0xC2DE9B41,  /*  45  */
  0x76F006B9,  /*  46  */
  0xDB393172,  /*  47  */
  0x679750DB,  /*  48  */
  0xCA655204,  /*  49  */
  0x79B1AA01,  /*  50  */
  0x6B7F1DB1,  /*  51  */
  0xAA834C35,  /*  52  */
  0x9B121507,  /*  53  */
  0x6959335A,  /*  54  */
  0xB66FC3D5,  /*  55  */
  0x1F7D56ED,  /*  56  */
  0xF274F294,  /*  57  */
  0x5D491602,  /*  58  */
  0x77B2DBE8,  /*  59  */
  0x3E92EF61,  /*  60  */
  0xE0D5DDA0,  /*  61  */
  0x14EEC07F,  /*  62  */
  0xBB0C3B3E,  /*  63  */
  0xBF9A46B8,  /*  64  */
  0xEE513ACF,  /*  65  */
  0xC577F6BC,  /*  66  */
  0xD162DFD9,  /*  67  */
  0x795B718,  /*  68  */
  0xEC1A3895,  /*  69  */
  0x68A81EB7,  /*  70  */
  0x7C3F3C47,  /*  71  */
  0x9D1FEC76,  /*  72  */
  0xC3B61132,  /*  73  */
  0xA733BDA9,  /*  74  */
  0x983AD434,  /*  75  */
  0x93D799BB,  /*  76  */
  0x9ED26670,  /*  77  */
  0xFADC1C,  /*  78  */
  0x56BE4843,  /*  79  */
  0x94CA491,  /*  80  */
  0x6FF2B9C4,  /*  81  */
  0xA293C933,  /*  82  */
  0x2D4228EB,  /*  83  */
  0x1EF4BB77,  /*  84  */
  0x69BB65C,  /*  85  */
  0xA9A02CDA,  /*  86  */
  0x8F1691D4,  /*  87  */
  0xB44F0E5E,  /*  88  */
  0x8CE7A52B,  /*  89  */
  0x9F61E45F,  /*  90  */
  0x492DEE16,  /*  91  */
  0xD9D927C5,  /*  92  */
  0xDA711A09,  /*  93  */
  0x47A92AC0,  /*  94  */
  0xA5227215,  /*  95  */
  0x1B10011A,  /*  96  */
  0xFAEA5F7D,  /*  97  */
  0x54195DE0,  /*  98  */
  0x90CE638D,  /*  99  */
  0xC4F798EA,  /*  100  */
  0x81D62F38,  /*  101  */
  0x1D14D521,  /*  102  */
  0xAE231465,  /*  103  */
  0x38D3E126,  /*  104  */
  0x353EA748,  /*  105  */
  0xE1C875D3,  /*  106  */
  0x7BBBB4E7,  /*  107  */
  0xFFBE50E,  /*  108  */
  0xEF289ABA,  /*  109  */
  0xBA997F7C,  /*  110  */
  0x50C2D887,  /*  111  */
  0xC95B909A,  /*  112  */
  0x6CB9FCF0,  /*  113  */
  0xDDE9D158,  /*  114  */
  0x62ED7156,  /*  115  */
  0x4550467,  /*  116  */
  0xEA3D807B,  /*  117  */
  0xE56AE7AD,  /*  118  */
  0xFF571C54,  /*  119  */
  0xE4C1D2B3,  /*  120  */
  0x152FA2FF,  /*  121  */
  0x2101A3F3,  /*  122  */
  0x7A26F9CC,  /*  123  */
  0xCFC91280,  /*  124  */
  0x2C67CBCA,  /*  125  */
  0xFC243224,  /*  126  */
  0xFE2A78D8,  /*  127  */
  0xB86937A8,  /*  128  */
  0x74A2F0A2,  /*  129  */
  0x99470D8B,  /*  130  */
  0x18E2C8C6,  /*  131  */
  0xF7607E0B,  /*  132  */
  0xF6ADBC85,  /*  133  */
  0x58C54E74,  /*  134  */
  0xCBF1C214,  /*  135  */
  0xE8FB14A,  /*  136  */
  0xC0B84500,  /*  137  */
  0xC8A75462,  /*  138  */
  0x645FCF5D,  /*  139  */
  0x4AB48466,  /*  140  */
  0xD6A3E3FE,  /*  141  */
  0x95F57A20,  /*  142  */
  0x4C4DB0A1,  /*  143  */
  0x430DC782,  /*  144  */
  0x85A444BD,  /*  145  */
  0x885E7DE9,  /*  146  */
  0xAC3CAFEC,  /*  147  */
  0x634B0A39,  /*  148  */
  0xC1EC3903,  /*  149  */
  0xD58AE2AB,  /*  150  */
  0x4FF65B71,  /*  151  */
  0xB381952F,  /*  152  */
  0x175347F6,  /*  153  */
  0xE7523F97,  /*  154  */
  0xE9F97C89,  /*  155  */
  0xDF9413DD,  /*  156  */
  0x72B3692C,  /*  157  */
  0x9740CA84,  /*  158  */
  0x80D170F7,  /*  159  */
  0xB4E3012,  /*  160  */
  0xA056B39C,  /*  161  */
  0xCC151BE1,  /*  162  */
  0x37E6CC60,  /*  163  */
  0xE29D6F52,  /*  164  */
  0xDBC8D3C,  /*  165  */
  0x3C1B188E,  /*  166  */
  0x39CA23AE,  /*  167  */
  0xD705B831,  /*  168  */
  0x89DCEA36,  /*  169  */
  0xA8F37763,  /*  170  */
  0x4D4A51EF,  /*  171  */
  0x5E02FDF2,  /*  172  */
  0x5F09FAFD,  /*  173  */
  0x36AA356C,  /*  174  */
  0x719EE617,  /*  175  */
  0x5AE19F1E,  /*  176  */
  0x3DDF4973,  /*  177  */
  0x4EB0AC83,  /*  178  */
  0x7F3597F4,  /*  179  */
  0x318EA153,  /*  180  */
  0x32D0F8A5,  /*  181  */
  0x16469319,  /*  182  */
  0xFDAB74CD,  /*  183  */
  0x1CFE8340,  /*  184  */
  0x26891F59,  /*  185  */
  0x517A87EE,  /*  186  */
  0x668740DE,  /*  187  */
  0x44FDAE4C,  /*  188  */
  0x55FFC61B,  /*  189  */
  0x70F88E90,  /*  190  */
  0xA103ED29,  /*  191  */
  0xB18B9DE6,  /*  192  */
  0xB9AF26C8,  /*  193  */
  0x128459F1,  /*  194  */
  0xCD29BF27,  /*  195  */
  0x4138039B,  /*  196  */
  0xF5ACAB05,  /*  197  */
  0x404443CB,  /*  198  */
  0xF882A010,  /*  199  */
  0x7572736A,  /*  200  */
  0x46508811,  /*  201  */
  0xD270214D,  /*  202  */
  0x61E40B55,  /*  203  */
  0x23905ED6,  /*  204  */
  0x60210C22,  /*  205  */
  0x290A8568,  /*  206  */
  0xD8C75CC9,  /*  207  */
  0xB591346D,  /*  208  */
  0x591EB5E5,  /*  209  */
  0x10252DAF,  /*  210  */
  0x96EB65DC,  /*  211  */
  0xA61C3699,  /*  212  */
  0x83186728,  /*  213  */
  0x11417664,  /*  214  */
  0xDEDAD7AC,  /*  215  */
  0x337B9CB0,  /*  216  */
  0x87BA2B8F,  /*  217  */
  0x735AF375,  /*  218  */
  0xC66BE857,  /*  219  */
  0xE3DD296B,  /*  220  */
  0x3445C14B,  /*  221  */
  0xC70E3D7E,  /*  222  */
  0x24FCE03B,  /*  223  */
  0x86E8EB08,  /*  224  */
  0x25CFFB93,  /*  225  */
  0xCED48A9F,  /*  226  */
  0xD36DF5B4,  /*  227  */
  0x3AD810C1,  /*  228  */
  0x570F82D0,  /*  229  */
  0x8EE3D9C7,  /*  230  */
  0xBD6CCD8C,  /*  231  */
  0x2B20DE98,  /*  232  */
  0xE8134AE3,  /*  233  */
  0x288042B2,  /*  234  */
  0x2AB7C530,  /*  235  */
  0x1A549237,  /*  236  */
  0xF3438106,  /*  237  */
  0x130BE6E,  /*  238  */
  0xF9004F5B,  /*  239  */
  0x5BBDF7D7,  /*  240  */
  0x3588B0D,  /*  241  */
  0xF12C05B5,  /*  242  */
  0x8DCDA842,  /*  243  */
  0x53360913,  /*  244  */
  0x9A64F178,  /*  245  */
  0x27066C51,  /*  246  */
  0x50457B6,  /*  247  */
  0x9CE0E91F,  /*  248  */
  0x5C759686,  /*  249  */
  0x659CCE0F,  /*  250  */
  0x827C240A,  /*  251  */
  0x3BC4253A,  /*  252  */
  0xED8DF4D1,  /*  253  */
  0x7D7922A4,  /*  254  */
  0x849F6BF5  /*  255  */
  /*  End of S Box 3  */ },


{  /*  Start of S Box 4  */
  0x3A13DB47,  /*  0  */
  0xFA73FED,  /*  1  */
  0xF59774B2,  /*  2  */
  0xA82D2E39,  /*  3  */
  0x1B01C668,  /*  4  */
  0xBA41E59E,  /*  5  */
  0x696D3AEE,  /*  6  */
  0x973F5F48,  /*  7  */
  0xE1E9E028,  /*  8  */
  0x90237B57,  /*  9  */
  0x6467E6C7,  /*  10  */
  0xE080FBA8,  /*  11  */
  0x3C771A13,  /*  12  */
  0xBF9DF688,  /*  13  */
  0x2C3EB876,  /*  14  */
  0xE4EAC142,  /*  15  */
  0xD8240C0F,  /*  16  */
  0x31CEFD5C,  /*  17  */
  0x8BF376DA,  /*  18  */
  0xCDEC51AB,  /*  19  */
  0x4D4919C6,  /*  20  */
  0xC35F2904,  /*  21  */
  0xEDBFEB08,  /*  22  */
  0xDD7C0A6F,  /*  23  */
  0x74D33B98,  /*  24  */
  0xD7D5B0C0,  /*  25  */
  0xE30084FD,  /*  26  */
  0xB861A6D5,  /*  27  */
  0x3BCC8C91,  /*  28  */
  0xBBB717A6,  /*  29  */
  0x575C5D74,  /*  30  */
  0xF1DDF1A4,  /*  31  */
  0x4B3D5C64,  /*  32  */
  0x1508DDBB,  /*  33  */
  0x76ED5317,  /*  34  */
  0x52E0C463,  /*  35  */
  0x10144197,  /*  36  */
  0xEB5BA202,  /*  37  */
  0x4160B1F0,  /*  38  */
  0xBC946A7C,  /*  39  */
  0xAC4B70D8,  /*  40  */
  0x4AB4A424,  /*  41  */
  0x9B6AC0AC,  /*  42  */
  0x35DCD9A5,  /*  43  */
  0xAA302BC9,  /*  44  */
  0xD6EBFEEB,  /*  45  */
  0x65EEAB7,  /*  46  */
  0x55B99035,  /*  47  */
  0x683A010E,  /*  48  */
  0x421F10D0,  /*  49  */
  0x21C6A980,  /*  50  */
  0xC86378C1,  /*  51  */
  0x7DEEB734,  /*  52  */
  0xB1C8301F,  /*  53  */
  0xC52F2D2C,  /*  54  */
  0x862BFF7D,  /*  55  */
  0xA9629441,  /*  56  */
  0xA3D97F59,  /*  57  */
  0x1718BD01,  /*  58  */
  0xC6835582,  /*  59  */
  0xB0FE00FF,  /*  60  */
  0x190C9F73,  /*  61  */
  0x5BD756EC,  /*  62  */
  0x8847F2D4,  /*  63  */
  0x11C4DF51,  /*  64  */
  0xB288068C,  /*  65  */
  0xAE2E28B3,  /*  66  */
  0x56B63610,  /*  67  */
  0x5A123EBA,  /*  68  */
  0x6E0F8906,  /*  69  */
  0x3BC6649,  /*  70  */
  0x28093C14,  /*  71  */
  0x7E488685,  /*  72  */
  0x4338E49F,  /*  73  */
  0xFBE71358,  /*  74  */
  0xD571E30C,  /*  75  */
  0x81612F1,  /*  76  */
  0x820AC972,  /*  77  */
  0x9999228F,  /*  78  */
  0xA633DE9A,  /*  79  */
  0xC7D2464C,  /*  80  */
  0x845273BF,  /*  81  */
  0x7B3616DB,  /*  82  */
  0x1FAF02AA,  /*  83  */
  0xB99CA0B4,  /*  84  */
  0xD1D4F7D9,  /*  85  */
  0xE7A3D060,  /*  86  */
  0x651B50F2,  /*  87  */
  0x7272AA69,  /*  88  */
  0xB3DA9753,  /*  89  */
  0xEFE40D1E,  /*  90  */
  0x9443779D,  /*  91  */
  0x33FA8F1A,  /*  92  */
  0x47BA8EDD,  /*  93  */
  0xF7899965,  /*  94  */
  0x3F5557C2,  /*  95  */
  0xC99E3526,  /*  96  */
  0x44AAD554,  /*  97  */
  0x1E10F3D6,  /*  98  */
  0xEE9A0BB6,  /*  99  */
  0x8D799DD2,  /*  100  */
  0xB5ACCF7E,  /*  101  */
  0x9F215E05,  /*  102  */
  0xD0542A5F,  /*  103  */
  0xF37DD6FE,  /*  104  */
  0x66916C36,  /*  105  */
  0x45D1E4A,  /*  106  */
  0x9C1EEF1B,  /*  107  */
  0x37577120,  /*  108  */
  0x803CCB19,  /*  109  */
  0x96CD593A,  /*  110  */
  0xF968A7CE,  /*  111  */
  0x22CB60EA,  /*  112  */
  0xEC7B6803,  /*  113  */
  0x26A41855,  /*  114  */
  0xD2A5CA15,  /*  115  */
  0xF4042FA2,  /*  116  */
  0x1806CD87,  /*  117  */
  0x9EABBEB1,  /*  118  */
  0x584095FC,  /*  119  */
  0x2AB84071,  /*  120  */
  0x38EFF83F,  /*  121  */
  0x61A1C812,  /*  122  */
  0x142A61C5,  /*  123  */
  0xFFA80F8A,  /*  124  */
  0x45118A9C,  /*  125  */
  0x29657262,  /*  126  */
  0xFDE62786,  /*  127  */
  0x50F8A50A,  /*  128  */
  0xD94F7CA1,  /*  129  */
  0xCA02E8C4,  /*  130  */
  0xF06FE9F5,  /*  131  */
  0x5FE81C0B,  /*  132  */
  0x858A5277,  /*  133  */
  0xB4E34DA7,  /*  134  */
  0xE9CFE7CA,  /*  135  */
  0x8AFCCCFA,  /*  136  */
  0x8FE17E50,  /*  137  */
  0xA7BDB97A,  /*  138  */
  0x530BC7DE,  /*  139  */
  0x818C67E2,  /*  140  */
  0xA2D8442D,  /*  141  */
  0x36461D93,  /*  142  */
  0x91F6982A,  /*  143  */
  0x1933D2B,  /*  144  */
  0x2D51BF38,  /*  145  */
  0xEA87B678,  /*  146  */
  0x398B7909,  /*  147  */
  0x5C2C0366,  /*  148  */
  0xCBAE31FB,  /*  149  */
  0x6A19343D,  /*  150  */
  0xFC1A42DF,  /*  151  */
  0x7856AF37,  /*  152  */
  0x7FFFACCC,  /*  153  */
  0x635026D1,  /*  154  */
  0x5D84FA46,  /*  155  */
  0xE2590ECD,  /*  156  */
  0x62F2F423,  /*  157  */
  0xB6BBBC32,  /*  158  */
  0x9D8F4C96,  /*  159  */
  0xC464374E,  /*  160  */
  0x4E69838D,  /*  161  */
  0x5E9BBB94,  /*  162  */
  0x2C1435A,  /*  163  */
  0xBEE52190,  /*  164  */
  0xB7DE246C,  /*  165  */
  0x40FB6D99,  /*  166  */
  0x6FADD84B,  /*  167  */
  0x3495E143,  /*  168  */
  0x6CB31B07,  /*  169  */
  0xCE255B5E,  /*  170  */
  0x798E49C3,  /*  171  */
  0x321D8B8E,  /*  172  */
  0x3D53C3E7,  /*  173  */
  0xD303D3E4,  /*  174  */
  0xDC0D936A,  /*  175  */
  0x30F56FDC,  /*  176  */
  0xC0A24ED7,  /*  177  */
  0x499809B9,  /*  178  */
  0x2774B4B5,  /*  179  */
  0xA1E29EE6,  /*  180  */
  0xA007DCE0,  /*  181  */
  0xE314AAD,  /*  182  */
  0x1D82E21C,  /*  183  */
  0x984E9B2F,  /*  184  */
  0xE87AAD25,  /*  185  */
  0x6D17EC2E,  /*  186  */
  0xAFF1116D,  /*  187  */
  0xC258CEA3,  /*  188  */
  0x2B45A161,  /*  189  */
  0x87FD45F9,  /*  190  */
  0xC1906944,  /*  191  */
  0x2EB1D275,  /*  192  */
  0x4CA0A392,  /*  193  */
  0x1CC2FC29,  /*  194  */
  0x6B8D1F6E,  /*  195  */
  0x4F4D7AC8,  /*  196  */
  0xDF5AF9E1,  /*  197  */
  0x93996F8,  /*  198  */
  0xDA9F87B0,  /*  199  */
  0xE6BE80E9,  /*  200  */
  0x73783340,  /*  201  */
  0x162665EF,  /*  202  */
  0x5428F545,  /*  203  */
  0x71CAB55D,  /*  204  */
  0xAD6B4B52,  /*  205  */
  0xF632B3E5,  /*  206  */
  0xD43BED3C,  /*  207  */
  0x92F9C2AE,  /*  208  */
  0xA57F6B1D,  /*  209  */
  0xA056E84,  /*  210  */
  0x20F7634D,  /*  211  */
  0x7CC7B230,  /*  212  */
  0x5359A33,  /*  213  */
  0x89761522,  /*  214  */
  0x12D1644F,  /*  215  */
  0xCC27A883,  /*  216  */
  0x48B08D18,  /*  217  */
  0x131CD431,  /*  218  */
  0x1A4CD789,  /*  219  */
  0xD705ABC,  /*  220  */
  0x25B23879,  /*  221  */
  0xE5F47D56,  /*  222  */
  0x246CBA27,  /*  223  */
  0x678182F7,  /*  224  */
  0x4644AB3B,  /*  225  */
  0x7A157595,  /*  226  */
  0x729540D,  /*  227  */
  0x23A985D3,  /*  228  */
  0x5166C55B,  /*  229  */
  0x854F6B,  /*  230  */
  0xB86DABD,  /*  231  */
  0x75F00711,  /*  232  */
  0xF2DFD1B8,  /*  233  */
  0xBDC5EEF4,  /*  234  */
  0x8EB5929B,  /*  235  */
  0xC9608AF,  /*  236  */
  0xDE7525CB,  /*  237  */
  0xFE3747E8,  /*  238  */
  0xABDB587F,  /*  239  */
  0x9AD048F3,  /*  240  */
  0xF842043E,  /*  241  */
  0x8C732070,  /*  242  */
  0x2F2291A9,  /*  243  */
  0xCFD60516,  /*  244  */
  0xFA34818B,  /*  245  */
  0x772014A0,  /*  246  */
  0x3E6E3200,  /*  247  */
  0x83A662E3,  /*  248  */
  0x707E9C81,  /*  249  */
  0x600E2C7B,  /*  250  */
  0xA44A88BE,  /*  251  */
  0x59C3F0F6,  /*  252  */
  0x95C92321,  /*  253  */
  0x939239CF,  /*  254  */
  0xDBC0AE67  /*  255  */
  /*  End of S Box 4  */ },


{  /*  Start of S Box 5  */
  0x7F19B458,  /*  0  */
  0xE0C3EBFB,  /*  1  */
  0x6D3705F2,  /*  2  */
  0x2F95D838,  /*  3  */
  0x5E6E1A99,  /*  4  */
  0xA1555BA3,  /*  5  */
  0x35ED5DB1,  /*  6  */
  0xE8B890EC,  /*  7  */
  0x3E2E0767,  /*  8  */
  0x1606476F,  /*  9  */
  0x848FE1AA,  /*  10  */
  0x8D0EA0C7,  /*  11  */
  0xF8298704,  /*  12  */
  0xFAE6BCBD,  /*  13  */
  0x97E159D8,  /*  14  */
  0x82EFB64C,  /*  15  */
  0xB0D076AF,  /*  16  */
  0x20724D94,  /*  17  */
  0x6B272980,  /*  18  */
  0x6F168F44,  /*  19  */
  0x24D63BD5,  /*  20  */
  0x6A3CA1A4,  /*  21  */
  0x3F4C8917,  /*  22  */
  0xB2A13556,  /*  23  */
  0xEF26CAC8,  /*  24  */
  0xEC02845F,  /*  25  */
  0xE6FEE650,  /*  26  */
  0x4E77387C,  /*  27  */
  0x142F85FC,  /*  28  */
  0xB16F3DC4,  /*  29  */
  0x54FCC6D6,  /*  30  */
  0x78B0A702,  /*  31  */
  0x4548D6F5,  /*  32  */
  0xA694BAD,  /*  33  */
  0x2BA21BE4,  /*  34  */
  0x680AE09B,  /*  35  */
  0x1E3AB1F8,  /*  36  */
  0x6E366CE6,  /*  37  */
  0x9F8D324F,  /*  38  */
  0x9B5E8C89,  /*  39  */
  0x395C06F0,  /*  40  */
  0xBB53DA09,  /*  41  */
  0xE763D196,  /*  42  */
  0xDA6A0AF1,  /*  43  */
  0x870E477,  /*  44  */
  0xCB405E22,  /*  45  */
  0xEB44B890,  /*  46  */
  0xD13E7442,  /*  47  */
  0x3656BD0,  /*  48  */
  0xC23B1E23,  /*  49  */
  0xD9DD8BDB,  /*  50  */
  0x43750EB7,  /*  51  */
  0xB7340EE,  /*  52  */
  0xACA44A07,  /*  53  */
  0x1DE2A6CA,  /*  54  */
  0x2AEC4E0E,  /*  55  */
  0xA52B4927,  /*  56  */
  0x615A7A72,  /*  57  */
  0xCD6201CF,  /*  58  */
  0xCEFFB3A8,  /*  59  */
  0x8EC7E735,  /*  60  */
  0x918E4254,  /*  61  */
  0xEE9F128C,  /*  62  */
  0x882008EA,  /*  63  */
  0xE21FF474,  /*  64  */
  0xDE5BEF6,  /*  65  */
  0x8FC4ABD7,  /*  66  */
  0xA7DB5830,  /*  67  */
  0x87C60D18,  /*  68  */
  0x7E9B0B4A,  /*  69  */
  0xB3AF3775,  /*  70  */
  0x8BD3730F,  /*  71  */
  0xD6B37088,  /*  72  */
  0x9A7B9655,  /*  73  */
  0xF2A830C1,  /*  74  */
  0x86964161,  /*  75  */
  0xD7ADFD3C,  /*  76  */
  0x23355C73,  /*  77  */
  0x94C1C25B,  /*  78  */
  0x1ABA5ADA,  /*  79  */
  0xD0F17D06,  /*  80  */
  0x9CF88081,  /*  81  */
  0x2E077586,  /*  82  */
  0xA91B036C,  /*  83  */
  0x119E7119,  /*  84  */
  0x80B90945,  /*  85  */
  0xC3D2B09D,  /*  86  */
  0xB5A0DD66,  /*  87  */
  0xE1C9FC1B,  /*  88  */
  0x3094A484,  /*  89  */
  0x2688EA8E,  /*  90  */
  0x92A763C0,  /*  91  */
  0xA62CF637,  /*  92  */
  0x2D5697EB,  /*  93  */
  0x5C66025E,  /*  94  */
  0x17F4672C,  /*  95  */
  0x25BBED11,  /*  96  */
  0x369152E8,  /*  97  */
  0xCC4E3FD4,  /*  98  */
  0xDDDED33D,  /*  99  */
  0xB4C0C34E,  /*  100  */
  0x44906D93,  /*  101  */
  0x50CF2DD2,  /*  102  */
  0x311C7EA6,  /*  103  */
  0x6728EE1E,  /*  104  */
  0x60E957CC,  /*  105  */
  0x37B4A346,  /*  106  */
  0xED09D53F,  /*  107  */
  0x4014B9AC,  /*  108  */
  0xAE41D047,  /*  109  */
  0x63112CBC,  /*  110  */
  0xF146B700,  /*  111  */
  0x66F29B2B,  /*  112  */
  0x6C85AE6E,  /*  113  */
  0xAB0478ED,  /*  114  */
  0x526B3EA0,  /*  115  */
  0x7B7CC0FE,  /*  116  */
  0x4FB260,  /*  117  */
  0xFECEBD34,  /*  118  */
  0xF959FBB8,  /*  119  */
  0xB947101F,  /*  120  */
  0x8CC239BA,  /*  121  */
  0xBA74158F,  /*  122  */
  0xF31E6936,  /*  123  */
  0xDCF53152,  /*  124  */
  0xE9DAC810,  /*  125  */
  0xF023D4CB,  /*  126  */
  0x3D30DBCD,  /*  127  */
  0x997E4FFA,  /*  128  */
  0xB8C5F382,  /*  129  */
  0xBF08E9EF,  /*  130  */
  0xE40B942F,  /*  131  */
  0x74BCDC97,  /*  132  */
  0xFF9979CE,  /*  133  */
  0x3CEBF179,  /*  134  */
  0x29A56068,  /*  135  */
  0xA2E46812,  /*  136  */
  0x2C5B2EBE,  /*  137  */
  0xFC867239,  /*  138  */
  0x1952AD71,  /*  139  */
  0x90B71C6D,  /*  140  */
  0xC6CDAC03,  /*  141  */
  0xC539BFA2,  /*  142  */
  0xF66020A5,  /*  143  */
  0xC84A647B,  /*  144  */
  0x69DC1691,  /*  145  */
  0x5951191D,  /*  146  */
  0xD4386670,  /*  147  */
  0x8576AF41,  /*  148  */
  0xE34D88A9,  /*  149  */
  0x72E7A87E,  /*  150  */
  0xD33192C6,  /*  151  */
  0x286C560D,  /*  152  */
  0x5D811F51,  /*  153  */
  0x542FE85,  /*  154  */
  0xC0B62321,  /*  155  */
  0x38A3817F,  /*  156  */
  0x5B0F279C,  /*  157  */
  0x898C62B4,  /*  158  */
  0xA3176F2E,  /*  159  */
  0x34AA9595,  /*  160  */
  0xE51DD29A,  /*  161  */
  0x21D5B5F4,  /*  162  */
  0xF5F324F7,  /*  163  */
  0x4880C4B2,  /*  164  */
  0x9D57A54B,  /*  165  */
  0xB600D7D3,  /*  166  */
  0xDFF6E29E,  /*  167  */
  0x1F9336F3,  /*  168  */
  0xAD7AD9B0,  /*  169  */
  0x753F340B,  /*  170  */
  0xA07F3CE9,  /*  171  */
  0x554B8313,  /*  172  */
  0xC7B12A3E,  /*  173  */
  0x81F7E332,  /*  174  */
  0xF124429,  /*  175  */
  0x7D250CB3,  /*  176  */
  0x96670F8A,  /*  177  */
  0x766133A1,  /*  178  */
  0x8AAB9D8B,  /*  179  */
  0xCA8AF06B,  /*  180  */
  0xDB13CCFF,  /*  181  */
  0xAF97E8DE,  /*  182  */
  0x41101DF9,  /*  183  */
  0x4B019125,  /*  184  */
  0x71DFF81A,  /*  185  */
  0x182DDE87,  /*  186  */
  0x4245F7DF,  /*  187  */
  0xDEEE7F31,  /*  188  */
  0x4FBDFA16,  /*  189  */
  0x19A5020,  /*  190  */
  0x9354464D,  /*  191  */
  0x151854E3,  /*  192  */
  0xFB649A62,  /*  193  */
  0x1CB22805,  /*  194  */
  0xBE848A83,  /*  195  */
  0x3B49CD76,  /*  196  */
  0x770DAA92,  /*  197  */
  0x732ABB24,  /*  198  */
  0xFDD19E98,  /*  199  */
  0xD2E80043,  /*  200  */
  0x4789A92D,  /*  201  */
  0x58587BC9,  /*  202  */
  0x490C3A78,  /*  203  */
  0x7C9D453A,  /*  204  */
  0x7998BA57,  /*  205  */
  0xA4B56A7D,  /*  206  */
  0x95E39FB9,  /*  207  */
  0x27AEC7D9,  /*  208  */
  0x98874CDC,  /*  209  */
  0x53AC7C59,  /*  210  */
  0x6405438D,  /*  211  */
  0xC4242149,  /*  212  */
  0x126D2226,  /*  213  */
  0x1B71CBC2,  /*  214  */
  0x338B86B6,  /*  215  */
  0xC992F9FD,  /*  216  */
  0x4CFBF564,  /*  217  */
  0xBC9C8D5A,  /*  218  */
  0x627D5FBB,  /*  219  */
  0xD53D2B14,  /*  220  */
  0x5F6898B5,  /*  221  */
  0x2221250A,  /*  222  */
  0xF4D965E1,  /*  223  */
  0x3AE048E0,  /*  224  */
  0xC1BF6E9F,  /*  225  */
  0x2A99C2A,  /*  226  */
  0x4DF9827A,  /*  227  */
  0xCF014E7,  /*  228  */
  0x13D7180C,  /*  229  */
  0x57322F63,  /*  230  */
  0x4650FFAB,  /*  231  */
  0xEFDC5E2,  /*  232  */
  0x61A13D1,  /*  233  */
  0x7FA55DD,  /*  234  */
  0xF7CCC1AE,  /*  235  */
  0x7A03EC48,  /*  236  */
  0x70781128,  /*  237  */
  0xCFCA9340,  /*  238  */
  0xBD345153,  /*  239  */
  0x9E838EBF,  /*  240  */
  0x6522613B,  /*  241  */
  0x565F5333,  /*  242  */
  0x1015CEC5,  /*  243  */
  0x4A79EFC3,  /*  244  */
  0x5AA6A25D,  /*  245  */
  0x9822615,  /*  246  */
  0xD8D87765,  /*  247  */
  0xB75D9908,  /*  248  */
  0xA8EADF5C,  /*  249  */
  0x51331769,  /*  250  */
  0xEACBCFA7,  /*  251  */
  0x4BEF26A,  /*  252  */
  0x32C80401,  /*  253  */
  0xAAD4C9E5,  /*  254  */
  0x8343E51C  /*  255  */
  /*  End of S Box 5  */ },


{  /*  Start of S Box 6  */
  0xB98FD895,  /*  0  */
  0xBD668A92,  /*  1  */
  0xB89ABB2E,  /*  2  */
  0xFA0A5642,  /*  3  */
  0xDE5B9B33,  /*  4  */
  0x1E376359,  /*  5  */
  0xF33AFD57,  /*  6  */
  0xA71EB7F9,  /*  7  */
  0x91AA4213,  /*  8  */
  0xE7CE2415,  /*  9  */
  0xD0A3670D,  /*  10  */
  0xCB80A1AE,  /*  11  */
  0xC43C6E9A,  /*  12  */
  0x34527F29,  /*  13  */
  0x9A9FDA8A,  /*  14  */
  0x1B88A685,  /*  15  */
  0xCF9E49D,  /*  16  */
  0xE26D6AA6,  /*  17  */
  0x1F4E4CEB,  /*  18  */
  0xA0903CC1,  /*  19  */
  0x7CCC43E7,  /*  20  */
  0x46200708,  /*  21  */
  0x76082BB6,  /*  22  */
  0xDAF51132,  /*  23  */
  0x7E26B08E,  /*  24  */
  0x21B5EE04,  /*  25  */
  0x3967D13E,  /*  26  */
  0x9DD846DC,  /*  27  */
  0x3717ACD9,  /*  28  */
  0x8A937A2C,  /*  29  */
  0x841F9452,  /*  30  */
  0x4B35146D,  /*  31  */
  0x5EF1EB70,  /*  32  */
  0x3B6B8490,  /*  33  */
  0x78A1BDBC,  /*  34  */
  0x1DAE9AB8,  /*  35  */
  0x75EA424,  /*  36  */
  0x2A423DFC,  /*  37  */
  0x7003B181,  /*  38  */
  0x9694D7B1,  /*  39  */
  0x949BF531,  /*  40  */
  0xE64D2745,  /*  41  */
  0xB591C6C2,  /*  42  */
  0x989C23B,  /*  43  */
  0x25ADB2DE,  /*  44  */
  0xFD1CA88B,  /*  45  */
  0xCF50374D,  /*  46  */
  0xD8166830,  /*  47  */
  0x52841F00,  /*  48  */
  0x3EB199DA,  /*  49  */
  0xFC8C1E26,  /*  50  */
  0xAD7AAFBA,  /*  51  */
  0xF938CCA7,  /*  52  */
  0x2912D5C0,  /*  53  */
  0x11253A79,  /*  54  */
  0x28F393ED,  /*  55  */
  0x267B10F8,  /*  56  */
  0xF406012A,  /*  57  */
  0xEA0F60F5,  /*  58  */
  0x43A83F20,  /*  59  */
  0x232821D,  /*  60  */
  0x83EA9893,  /*  61  */
  0x352A890C,  /*  62  */
  0xE8DA5734,  /*  63  */
  0x7A295DE6,  /*  64  */
  0x44EE6C0F,  /*  65  */
  0x7D63F1F1,  /*  66  */
  0x5A4C3E62,  /*  67  */
  0xCE925036,  /*  68  */
  0x81F7CF09,  /*  69  */
  0x54DE17D4,  /*  70  */
  0x4DFD0D63,  /*  71  */
  0x3FB2022,  /*  72  */
  0x73C19676,  /*  73  */
  0x8E414782,  /*  74  */
  0x3D5A8C53,  /*  75  */
  0xE3CD09A1,  /*  76  */
  0x42E2EFD,  /*  77  */
  0x19570EB0,  /*  78  */
  0xC3E4FBEE,  /*  79  */
  0xAB148384,  /*  80  */
  0xB2BC1205,  /*  81  */
  0x4E5C778F,  /*  82  */
  0x5B97DFF2,  /*  83  */
  0xE4779DFB,  /*  84  */
  0xFB59A255,  /*  85  */
  0xF769E69,  /*  86  */
  0x41624950,  /*  87  */
  0xF810E1D3,  /*  88  */
  0x851519D7,  /*  89  */
  0x2F54B97E,  /*  90  */
  0x93E7855E,  /*  91  */
  0x599DAD4C,  /*  92  */
  0x2D58F07F,  /*  93  */
  0xD1CA26A0,  /*  94  */
  0xD2BC9EF,  /*  95  */
  0x458E36A5,  /*  96  */
  0x68409CB3,  /*  97  */
  0x6F7EE7A2,  /*  98  */
  0xF6CFB3A8,  /*  99  */
  0xEDFF3A4,  /*  100  */
  0x2CC372CE,  /*  101  */
  0x9F69DB49,  /*  102  */
  0x63A0E87D,  /*  103  */
  0x7B5DB888,  /*  104  */
  0xC5477171,  /*  105  */
  0x36055F60,  /*  106  */
  0xA6E51516,  /*  107  */
  0x3171FF98,  /*  108  */
  0x12F44DC8,  /*  109  */
  0xCAD7751F,  /*  110  */
  0xDFACB6CD,  /*  111  */
  0x7F310AB7,  /*  112  */
  0xD56C38E3,  /*  113  */
  0x66D041C5,  /*  114  */
  0x6A180BF0,  /*  115  */
  0x833FAEA,  /*  116  */
  0x47EF1A6F,  /*  117  */
  0x16002174,  /*  118  */
  0xA8E15C77,  /*  119  */
  0xF52F30CA,  /*  120  */
  0xD621E99F,  /*  121  */
  0xE13F51B9,  /*  122  */
  0x8BF67DF7,  /*  123  */
  0x487D9510,  /*  124  */
  0x4298ECBD,  /*  125  */
  0x1DDA7FF,  /*  126  */
  0xBA3E083C,  /*  127  */
  0xDD3D32D8,  /*  128  */
  0xE5B6D61A,  /*  129  */
  0x1CDCD2E4,  /*  130  */
  0x50D2D975,  /*  131  */
  0x53FFAAF6,  /*  132  */
  0xAF687BB4,  /*  133  */
  0x6595293A,  /*  134  */
  0x5160C7DD,  /*  135  */
  0x5DF286AB,  /*  136  */
  0xD2A297C9,  /*  137  */
  0x13D5BAE1,  /*  138  */
  0x156F7007,  /*  139  */
  0xA1B28BB,  /*  140  */
  0x88B04F9C,  /*  141  */
  0xDC274AE9,  /*  142  */
  0x234964E8,  /*  143  */
  0x40A92D17,  /*  144  */
  0x6C9CB561,  /*  145  */
  0x5CE68106,  /*  146  */
  0xBFE4037,  /*  147  */
  0xBFB78B4B,  /*  148  */
  0x67CBDE41,  /*  149  */
  0xB0012519,  /*  150  */
  0xC17804D1,  /*  151  */
  0xEB451C97,  /*  152  */
  0x69B2FC28,  /*  153  */
  0x33E8556A,  /*  154  */
  0xCD099265,  /*  155  */
  0x9579C49B,  /*  156  */
  0xE9C48ECB,  /*  157  */
  0xBB4A9F8D,  /*  158  */
  0xB16EEA73,  /*  159  */
  0xD773F958,  /*  160  */
  0x6011D33F,  /*  161  */
  0x9E87A354,  /*  162  */
  0x182CB486,  /*  163  */
  0x98225296,  /*  164  */
  0xA46AE5FA,  /*  165  */
  0x3C618D5F,  /*  166  */
  0xBC815340,  /*  167  */
  0x3AD4BF66,  /*  168  */
  0xEC707EE0,  /*  169  */
  0x62347C1B,  /*  170  */
  0x566406C7,  /*  171  */
  0xC0AB5A51,  /*  172  */
  0x1A1ADC18,  /*  173  */
  0xAEC945A9,  /*  174  */
  0x10D1915B,  /*  175  */
  0x6D23CB0E,  /*  176  */
  0xA5FCCEB5,  /*  177  */
  0xFFBE4899,  /*  178  */
  0xFEBBDD1C,  /*  179  */
  0xB60E61EC,  /*  180  */
  0x279E447B,  /*  181  */
  0x570BA0BF,  /*  182  */
  0x77078778,  /*  183  */
  0x6BC01638,  /*  184  */
  0x4C4B1802,  /*  185  */
  0xF0A48087,  /*  186  */
  0x8C3BE60A,  /*  187  */
  0xD3E25467,  /*  188  */
  0x55741D0B,  /*  189  */
  0x714FC59E,  /*  190  */
  0x4A0CE2AF,  /*  191  */
  0x9B8D35DF,  /*  192  */
  0x49D33143,  /*  193  */
  0x38BD7603,  /*  194  */
  0x6EEDF794,  /*  195  */
  0x24485823,  /*  196  */
  0x89D9884F,  /*  197  */
  0x20F0628C,  /*  198  */
  0x82F8ABAA,  /*  199  */
  0xDB28D056,  /*  200  */
  0xEF191BE2,  /*  201  */
  0xEDC82F4E,  /*  202  */
  0xF2EB2AD5,  /*  203  */
  0x9CD60C39,  /*  204  */
  0xAA55595C,  /*  205  */
  0xBE86BE12,  /*  206  */
  0xA9BFF272,  /*  207  */
  0xE0A5FE5A,  /*  208  */
  0x5F824B46,  /*  209  */
  0xC256C114,  /*  210  */
  0xF753786C,  /*  211  */
  0x3013D4FE,  /*  212  */
  0x79A7652B,  /*  213  */
  0x327FF64A,  /*  214  */
  0xC7C5E07A,  /*  215  */
  0x99C64E68,  /*  216  */
  0x2265A92D,  /*  217  */
  0x2E7C6BC3,  /*  218  */
  0x61E90247,  /*  219  */
  0x97E08FCC,  /*  220  */
  0x80C774D6,  /*  221  */
  0xB436A5F3,  /*  222  */
  0x8FC239A3,  /*  223  */
  0xAC46F825,  /*  224  */
  0x6DB79C4,  /*  225  */
  0x758305D0,  /*  226  */
  0xF1A6E37C,  /*  227  */
  0xB799BCB2,  /*  228  */
  0xD4305E6E,  /*  229  */
  0x90B83B11,  /*  230  */
  0xC60266DB,  /*  231  */
  0x92AFCA91,  /*  232  */
  0x648AC06B,  /*  233  */
  0xCC516948,  /*  234  */
  0xA14323F4,  /*  235  */
  0x58447321,  /*  236  */
  0x3F0D5B64,  /*  237  */
  0x7204CD83,  /*  238  */
  0xD9721389,  /*  239  */
  0xC9B42CCF,  /*  240  */
  0xA375EF80,  /*  241  */
  0xA2EC6D35,  /*  242  */
  0x17BAC32F,  /*  243  */
  0x87E303E5,  /*  244  */
  0x148534BE,  /*  245  */
  0xB9AE27,  /*  246  */
  0x4F1D903D,  /*  247  */
  0xEE39F45D,  /*  248  */
  0x52DEDD2,  /*  249  */
  0x745F6F44,  /*  250  */
  0x86FAC801,  /*  251  */
  0x8DB322AC,  /*  252  */
  0xB3240FC6,  /*  253  */
  0x2B9633AD,  /*  254  */
  0xC88B001E  /*  255  */
  /*  End of S Box 6  */ },


{  /*  Start of S Box 7  */
  0xD93B928F,  /*  0  */
  0x15FCCA5F,  /*  1  */
  0x1FEEC53E,  /*  2  */
  0x780A03EB,  /*  3  */
  0xC2EA8AD0,  /*  4  */
  0x5465EE0D,  /*  5  */
  0x483FEDAA,  /*  6  */
  0x9DAF59F1,  /*  7  */
  0xB877B015,  /*  8  */
  0x595C8F9B,  /*  9  */
  0x7CB671DB,  /*  10  */
  0xEB800ECB,  /*  11  */
  0x421C47A,  /*  12  */
  0x2B97B8C,  /*  13  */
  0xBAB4D553,  /*  14  */
  0xADC745BA,  /*  15  */
  0x7DDAF8B1,  /*  16  */
  0xD5CFB16C,  /*  17  */
  0x5338AF82,  /*  18  */
  0xD0478D81,  /*  19  */
  0x8D9B344D,  /*  20  */
  0x44074BC5,  /*  21  */
  0x4CC51188,  /*  22  */
  0xF6DA7,  /*  23  */
  0xBBE44DD7,  /*  24  */
  0x9C4BD055,  /*  25  */
  0xB062B296,  /*  26  */
  0x5D416BAC,  /*  27  */
  0x9334EBBF,  /*  28  */
  0x8A9FA94F,  /*  29  */
  0xD6255C6A,  /*  30  */
  0x7BF588E4,  /*  31  */
  0xA64E533C,  /*  32  */
  0xB958FAF6,  /*  33  */
  0x56D51E9D,  /*  34  */
  0x5275E669,  /*  35  */
  0x87E6A8A2,  /*  36  */
  0x5CC01061,  /*  37  */
  0xBFF22B60,  /*  38  */
  0xC811BAC4,  /*  39  */
  0xAC1A6832,  /*  40  */
  0xA1B3D459,  /*  41  */
  0xE9C2138E,  /*  42  */
  0x1B18EC64,  /*  43  */
  0xC6FD1422,  /*  44  */
  0xDA989842,  /*  45  */
  0xFD303658,  /*  46  */
  0xF7DD9FA0,  /*  47  */
  0xC3932695,  /*  48  */
  0x9F81ABAE,  /*  49  */
  0x603962C0,  /*  50  */
  0x64166949,  /*  51  */
  0xA0370B18,  /*  52  */
  0x4FF438D2,  /*  53  */
  0x996D1F00,  /*  54  */
  0x3D64DB4C,  /*  55  */
  0x1E675AD6,  /*  56  */
  0xCFD01D47,  /*  57  */
  0x901E3F57,  /*  58  */
  0xC9A541E9,  /*  59  */
  0x92088213,  /*  60  */
  0x756FFF3A,  /*  61  */
  0x21510AB8,  /*  62  */
  0x8357A4B9,  /*  63  */
  0x17F3DE8B,  /*  64  */
  0x34712730,  /*  65  */
  0xDEBCE78A,  /*  66  */
  0xBE2D5E20,  /*  67  */
  0xCEF7C039,  /*  68  */
  0x97FBF79F,  /*  69  */
  0xBF96197,  /*  70  */
  0x86824768,  /*  71  */
  0xAEF8350B,  /*  72  */
  0x2988E1EF,  /*  73  */
  0x43F02D34,  /*  74  */
  0x4B3E8E78,  /*  75  */
  0xD74A676D,  /*  76  */
  0x7F76F0BE,  /*  77  */
  0x2FD31CFD,  /*  78  */
  0x7073D97C,  /*  79  */
  0x11F000F,  /*  80  */
  0x6E9AB3C6,  /*  81  */
  0xB66B7A09,  /*  82  */
  0x4D1D17AF,  /*  83  */
  0x57BF312D,  /*  84  */
  0x72A277FF,  /*  85  */
  0x4229568D,  /*  86  */
  0x5F50EA26,  /*  87  */
  0x1CB0E854,  /*  88  */
  0xEA95E9BB,  /*  89  */
  0x2DEFE340,  /*  90  */
  0x207BB41B,  /*  91  */
  0x25CAFC2E,  /*  92  */
  0x14C4B602,  /*  93  */
  0xF17906BD,  /*  94  */
  0x3B52A135,  /*  95  */
  0xA89A094,  /*  96  */
  0xECB77523,  /*  97  */
  0xF6C8A648,  /*  98  */
  0x6F708BB4,  /*  99  */
  0x3CFE1AE1,  /*  100  */
  0x5361680,  /*  101  */
  0x58D23767,  /*  102  */
  0x396A7373,  /*  103  */
  0x504515CF,  /*  104  */
  0x802A40A4,  /*  105  */
  0x45409CD8,  /*  106  */
  0x18E7707D,  /*  107  */
  0x6BBBD2E0,  /*  108  */
  0xD290095E,  /*  109  */
  0xB3B22117,  /*  110  */
  0x66CD843F,  /*  111  */
  0x89047FB0,  /*  112  */
  0xA98F3E71,  /*  113  */
  0xFAA1D1DE,  /*  114  */
  0xD26B96E,  /*  115  */
  0x5159CC04,  /*  116  */
  0xF0C3CEFA,  /*  117  */
  0x3F0E6F14,  /*  118  */
  0xFC9404C1,  /*  119  */
  0xF20395E7,  /*  120  */
  0x11AB97ED,  /*  121  */
  0x76C1CD19,  /*  122  */
  0x2620199E,  /*  123  */
  0x302B0711,  /*  124  */
  0x36FFFBCE,  /*  125  */
  0xCB7D3212,  /*  126  */
  0x8878A2AB,  /*  127  */
  0xAB24579C,  /*  128  */
  0x388D12DF,  /*  129  */
  0xDDD89601,  /*  130  */
  0x63C6AC90,  /*  131  */
  0x22E1BE05,  /*  132  */
  0x3EAA054A,  /*  133  */
  0x9E5483A1,  /*  134  */
  0x1AD10CD5,  /*  135  */
  0x6A1948B5,  /*  136  */
  0x4A9E6C52,  /*  137  */
  0xB1A9C843,  /*  138  */
  0x943120AD,  /*  139  */
  0x19DBE5FB,  /*  140  */
  0xAAA7661D,  /*  141  */
  0xF40624B2,  /*  142  */
  0xF5AE9056,  /*  143  */
  0xCA1C65CA,  /*  144  */
  0x35B7D46,  /*  145  */
  0xA72239E8,  /*  146  */
  0x72E3C29,  /*  147  */
  0x4E842F75,  /*  148  */
  0xBCACA303,  /*  149  */
  0x162C9A4B,  /*  150  */
  0xBDF644F3,  /*  151  */
  0x6C35DC86,  /*  152  */
  0x2E92F536,  /*  153  */
  0xEF4F2537,  /*  154  */
  0x6E86EF9,  /*  155  */
  0x8F68A5E5,  /*  156  */
  0xD35A426B,  /*  157  */
  0x5BF15B50,  /*  158  */
  0xE33DBD31,  /*  159  */
  0xC71BAAB7,  /*  160  */
  0x77E5D76F,  /*  161  */
  0x98CB4A91,  /*  162  */
  0x27EC28F0,  /*  163  */
  0x9A17D3A6,  /*  164  */
  0x68A3870E,  /*  165  */
  0x3515FD77,  /*  166  */
  0x914D3AC3,  /*  167  */
  0x339DE425,  /*  168  */
  0x84FABCEC,  /*  169  */
  0xF9995F10,  /*  170  */
  0xEA83D27,  /*  171  */
  0xFBB1897E,  /*  172  */
  0xDB28552A,  /*  173  */
  0xC561C3E3,  /*  174  */
  0xE710C993,  /*  175  */
  0x326EDF51,  /*  176  */
  0x1043D8D1,  /*  177  */
  0x239C0DA8,  /*  178  */
  0x2B55BB08,  /*  179  */
  0x2C0CDD5C,  /*  180  */
  0x28913B5D,  /*  181  */
  0x553C0F45,  /*  182  */
  0xC14464CC,  /*  183  */
  0x40AD79A9,  /*  184  */
  0x4101937F,  /*  185  */
  0x3A975D87,  /*  186  */
  0xC63C144,  /*  187  */
  0x82EBB5C9,  /*  188  */
  0x738785F2,  /*  189  */
  0xD4695466,  /*  190  */
  0xC0A6C6A5,  /*  191  */
  0xB77A5838,  /*  192  */
  0x7AD92ADA,  /*  193  */
  0xB4134641,  /*  194  */
  0x69857862,  /*  195  */
  0xE88B8CF7,  /*  196  */
  0x71BD7C33,  /*  197  */
  0x5E0D9EBC,  /*  198  */
  0xED668192,  /*  199  */
  0x318C02FE,  /*  200  */
  0xA8A46306,  /*  201  */
  0xCD0B8070,  /*  202  */
  0x8B84FF5,  /*  203  */
  0x4912EFEA,  /*  204  */
  0xF324C21,  /*  205  */
  0x5A235299,  /*  206  */
  0xC474FEF8,  /*  207  */
  0x6DD71B2B,  /*  208  */
  0xE5CE081F,  /*  209  */
  0xDC46B72C,  /*  210  */
  0xE086F1DC,  /*  211  */
  0x7E4C915A,  /*  212  */
  0x13DC5072,  /*  213  */
  0x8133F376,  /*  214  */
  0xA3D6F4D4,  /*  215  */
  0x95DE6A1A,  /*  216  */
  0xE1429DE2,  /*  217  */
  0xA427F283,  /*  218  */
  0x79E2863D,  /*  219  */
  0x8BD4E007,  /*  220  */
  0xCC7FB824,  /*  221  */
  0xF3CC519A,  /*  222  */
  0xE6DFF984,  /*  223  */
  0xB5962274,  /*  224  */
  0xB27E721C,  /*  225  */
  0x3709491E,  /*  226  */
  0x24B5C7B3,  /*  227  */
  0x9614AE98,  /*  228  */
  0x6256DACD,  /*  229  */
  0xA5BAC2C2,  /*  230  */
  0xF853BFD9,  /*  231  */
  0xFF602CDD,  /*  232  */
  0xD800F665,  /*  233  */
  0x47EDCF28,  /*  234  */
  0x9B6C4363,  /*  235  */
  0x8502CB79,  /*  236  */
  0xE248A7C7,  /*  237  */
  0xEE057EE6,  /*  238  */
  0x655D23F4,  /*  239  */
  0x1DBED63B,  /*  240  */
  0xFE8329B6,  /*  241  */
  0x8C7C990C,  /*  242  */
  0xAF499BFC,  /*  243  */
  0xDF72742F,  /*  244  */
  0x123A30A3,  /*  245  */
  0xD12F76EE,  /*  246  */
  0xE48EAD16,  /*  247  */
  0x46A0E20A,  /*  248  */
  0x618A945B,  /*  249  */
  0x9E3604E,  /*  250  */
  0xA2C92ED3,  /*  251  */
  0x745F4E7B,  /*  252  */
  0x675E3385,  /*  253  */
  0x8EE918C8,  /*  254  */
  0x2AE00189  /*  255  */
  /*  End of S Box 7  */ },


{  /*  Start of S Box 8  */
  0xE43BC2FC,  /*  0  */
  0x1229ED8E,  /*  1  */
  0x4F743AF7,  /*  2  */
  0xA1D90A44,  /*  3  */
  0xD51F3652,  /*  4  */
  0x6C2B5151,  /*  5  */
  0x48456E63,  /*  6  */
  0x608E1EE6,  /*  7  */
  0xCB68533F,  /*  8  */
  0xDF3FFB6,  /*  9  */
  0x9806480E,  /*  10  */
  0xD4FC0B8A,  /*  11  */
  0x6744FD20,  /*  12  */
  0x5A41439,  /*  13  */
  0xAF81F716,  /*  14  */
  0x6142D23B,  /*  15  */
  0x86BEAB77,  /*  16  */
  0x24804C36,  /*  17  */
  0x347931F4,  /*  18  */
  0xF94F39E9,  /*  19  */
  0xAD0FC8D4,  /*  20  */
  0xEC12FABD,  /*  21  */
  0xD80D62FF,  /*  22  */
  0x78701FFD,  /*  23  */
  0x6676D69A,  /*  24  */
  0x9E8D43A3,  /*  25  */
  0x13579784,  /*  26  */
  0x2F954559,  /*  27  */
  0x58AF1273,  /*  28  */
  0x1499DD05,  /*  29  */
  0x1AFAE57C,  /*  30  */
  0xF7FA44D,  /*  31  */
  0x9CCA09E,  /*  32  */
  0x30A09955,  /*  33  */
  0x51FFF0DC,  /*  34  */
  0xBE94927D,  /*  35  */
  0x183111BB,  /*  36  */
  0x4E260EDA,  /*  37  */
  0x97D78392,  /*  38  */
  0x32522899,  /*  39  */
  0xF47C3BCD,  /*  40  */
  0xFFE7B89F,  /*  41  */
  0xE8AB9B7E,  /*  42  */
  0x6F8A4BCF,  /*  43  */
  0x2BE5CDA0,  /*  44  */
  0xD1B1F9A9,  /*  45  */
  0x7A16C13E,  /*  46  */
  0xB269DC5B,  /*  47  */
  0xB072D90A,  /*  48  */
  0xC6A5F5E5,  /*  49  */
  0xDBB9A2F1,  /*  50  */
  0x7F25698C,  /*  51  */
  0x82B26702,  /*  52  */
  0x72EA8FD3,  /*  53  */
  0xB6DF3C33,  /*  54  */
  0xBF202C2E,  /*  55  */
  0xE5515296,  /*  56  */
  0xF50081D7,  /*  57  */
  0x8F3C9E62,  /*  58  */
  0xC4871CEB,  /*  59  */
  0xB1BB1372,  /*  60  */
  0x46C87A23,  /*  61  */
  0xEB14B204,  /*  62  */
  0x876F9D74,  /*  63  */
  0x420808AD,  /*  64  */
  0x8D4DFCF2,  /*  65  */
  0x93F122C0,  /*  66  */
  0x555BE932,  /*  67  */
  0x5BB06B4E,  /*  68  */
  0x4190ACC9,  /*  69  */
  0xA922754A,  /*  70  */
  0x10361AB2,  /*  71  */
  0xCEED636E,  /*  72  */
  0x7418C40C,  /*  73  */
  0xBBDE01FE,  /*  74  */
  0x11AA09F5,  /*  75  */
  0x52070491,  /*  76  */
  0x73B6666A,  /*  77  */
  0x62CB8690,  /*  78  */
  0x361920D9,  /*  79  */
  0x164CA1A1,  /*  80  */
  0xFD6CEA08,  /*  81  */
  0x1D3D823C,  /*  82  */
  0x3EC96AAB,  /*  83  */
  0x4A1B7380,  /*  84  */
  0x6A1A0F67,  /*  85  */
  0xA05DE7FB,  /*  86  */
  0x83E876E7,  /*  87  */
  0x759B5C8F,  /*  88  */
  0x3728062A,  /*  89  */
  0x7ED64630,  /*  90  */
  0x20B416CC,  /*  91  */
  0x40D3B6C4,  /*  92  */
  0xBBCE834,  /*  93  */
  0xC7B82454,  /*  94  */
  0x351C9C4F,  /*  95  */
  0x8A1F409,  /*  96  */
  0xFC7B93D0,  /*  97  */
  0x31408882,  /*  98  */
  0xB59FBAAA,  /*  99  */
  0xF6750349,  /*  100  */
  0x5CD85BEF,  /*  101  */
  0xD2AD960F,  /*  102  */
  0x394BB9E0,  /*  103  */
  0x5D9E5693,  /*  104  */
  0xBDBACFDB,  /*  105  */
  0x765AF321,  /*  106  */
  0xEAEEF6B4,  /*  107  */
  0xAC98A831,  /*  108  */
  0x2562218D,  /*  109  */
  0x5FC6405D,  /*  110  */
  0xE7FDEBD8,  /*  111  */
  0xF2B777C5,  /*  112  */
  0x57B52713,  /*  113  */
  0x5666C09C,  /*  114  */
  0xC2F41B3,  /*  115  */
  0x3DE115E8,  /*  116  */
  0x1CCD6C41,  /*  117  */
  0x68678D1E,  /*  118  */
  0x5458F271,  /*  119  */
  0x8A39E440,  /*  120  */
  0xBA7AAF24,  /*  121  */
  0xDE1E711F,  /*  122  */
  0x69EB4406,  /*  123  */
  0x2A96336C,  /*  124  */
  0x9954385E,  /*  125  */
  0xAAE226BA,  /*  126  */
  0x92382E60,  /*  127  */
  0x91E957B0,  /*  128  */
  0x8E888CF9,  /*  129  */
  0x17837218,  /*  130  */
  0x1E853F12,  /*  131  */
  0x44D5BFC3,  /*  132  */
  0xFAA85961,  /*  133  */
  0xE35E78D5,  /*  134  */
  0xC92E3410,  /*  135  */
  0x9A6A5DA2,  /*  136  */
  0xC88B4F86,  /*  137  */
  0x6EBDAD1A,  /*  138  */
  0x54975,  /*  139  */
  0x94F0A5BC,  /*  140  */
  0x23DB6F07,  /*  141  */
  0x27498498,  /*  142  */
  0x7013A64C,  /*  143  */
  0xED5FE3AE,  /*  144  */
  0x282CCC83,  /*  145  */
  0x77F47B88,  /*  146  */
  0xE6CF29DE,  /*  147  */
  0xDDE6D146,  /*  148  */
  0xC33EAA2C,  /*  149  */
  0xF0350DC7,  /*  150  */
  0xD6332AD2,  /*  151  */
  0xE0532343,  /*  152  */
  0x64022BF3,  /*  153  */
  0x327DB94,  /*  154  */
  0x636D05EC,  /*  155  */
  0x2D8461ED,  /*  156  */
  0x66EFEB5,  /*  157  */
  0x3F1760D1,  /*  158  */
  0xAEC7B501,  /*  159  */
  0xF8591797,  /*  160  */
  0x3BA75E03,  /*  161  */
  0xC06B1D26,  /*  162  */
  0x659C425F,  /*  163  */
  0x38A9192B,  /*  164  */
  0x4BF7BB22,  /*  165  */
  0x882D6427,  /*  166  */
  0xF724D7BE,  /*  167  */
  0xDC0B65AC,  /*  168  */
  0xA83790E2,  /*  169  */
  0xCF5C473A,  /*  170  */
  0xD9099117,  /*  171  */
  0x15D05AC8,  /*  172  */
  0xB365D5D6,  /*  173  */
  0xC201CEB7,  /*  174  */
  0x45C3DF58,  /*  175  */
  0x95C48BE1,  /*  176  */
  0x2C862D0B,  /*  177  */
  0xA256559D,  /*  178  */
  0x964AD376,  /*  179  */
  0x9D9DE689,  /*  180  */
  0x498C32A8,  /*  181  */
  0xAB73B368,  /*  182  */
  0x2FEE0A5,  /*  183  */
  0x7BC110E3,  /*  184  */
  0x4382A748,  /*  185  */
  0x590E4DAF,  /*  186  */
  0xA70A30EE,  /*  187  */
  0x47AE94C1,  /*  188  */
  0x33557D53,  /*  189  */
  0xFE1D35FA,  /*  190  */
  0x1F9A50BF,  /*  191  */
  0x7DF8E128,  /*  192  */
  0x6B30CB5A,  /*  193  */
  0x9B0318CB,  /*  194  */
  0xF3A26D81,  /*  195  */
  0x53217450,  /*  196  */
  0x73A9F19,  /*  197  */
  0x85604A38,  /*  198  */
  0xF1F67CA6,  /*  199  */
  0x9F4EAE56,  /*  200  */
  0x7CEFEC79,  /*  201  */
  0x115EE70,  /*  202  */
  0xFBCA25B9,  /*  203  */
  0xD732E225,  /*  204  */
  0x4D47C6CA,  /*  205  */
  0xEE40747,  /*  206  */
  0x2E502FEA,  /*  207  */
  0xB4777FF6,  /*  208  */
  0xAFBD064,  /*  209  */
  0x5E973E1C,  /*  210  */
  0x4717E15,  /*  211  */
  0xCDDAD8A7,  /*  212  */
  0xA4DD5469,  /*  213  */
  0xDF61C314,  /*  214  */
  0x84C5C766,  /*  215  */
  0x22AC683D,  /*  216  */
  0x80239A6B,  /*  217  */
  0x71D102F8,  /*  218  */
  0xDA7E9587,  /*  219  */
  0x4C89C9DF,  /*  220  */
  0xB70C0085,  /*  221  */
  0xCAB3D411,  /*  222  */
  0x19A63D57,  /*  223  */
  0x9CEC807B,  /*  224  */
  0x3CDCDADD,  /*  225  */
  0x8CE08A35,  /*  226  */
  0xD093A98B,  /*  227  */
  0x5A784E45,  /*  228  */
  0xE13F87B8,  /*  229  */
  0xC548F829,  /*  230  */
  0x817DBD6F,  /*  231  */
  0xBCBF707F,  /*  232  */
  0x790479E4,  /*  233  */
  0x3A11B437,  /*  234  */
  0x89CE376D,  /*  235  */
  0xCC10894B,  /*  236  */
  0x21F5A365,  /*  237  */
  0xC192BC78,  /*  238  */
  0xA3A30C9B,  /*  239  */
  0xB9C0B12D,  /*  240  */
  0xE92AF100,  /*  241  */
  0xA534581D,  /*  242  */
  0xA6F9FBCE,  /*  243  */
  0x8B41CA1B,  /*  244  */
  0x50D41BC2,  /*  245  */
  0xEFE38E5C,  /*  246  */
  0x29C25F0D,  /*  247  */
  0xB864EFC6,  /*  248  */
  0x9063B77A,  /*  249  */
  0x6D4398B1,  /*  250  */
  0xEE91DE95,  /*  251  */
  0x1B46B0A4,  /*  252  */
  0xD3D2C52F,  /*  253  */
  0x26F2BEF0,  /*  254  */
  0xE28F8542  /*  255  */
  /*  End of S Box 8  */ }


};
